@@ -8,7 +8,10 @@
 #       This module contains routines used by the Owl scripts.
 #
 # Revision History:
-#	1.0	121201	Initial version.
+#	1.0	Initial version.				121201
+#
+#	2.0	Released as part of DNSSEC-Tools 2.0		131301
+#	2.0.1	Modified to allow multiple types of queries.	131319
 #
 
 package owlutils;
@@ -24,7 +27,8 @@ use Log::Dispatch::FileRotate;
 # Version information.
 #
 my $NAME   = 'owlutils.pm';
-my $VERS   = "$NAME version: 2.0";
+my $VERS   = "$NAME version: 2.0.1";
+my $DTVERS = "DNSSEC-Tools version: 2.0";
 
 our @ISA = qw(Exporter);
 
@@ -38,6 +42,7 @@ our @EXPORT = qw(
 			owl_chkdir
 			owl_getpid
 			owl_halt
+			owl_log
 			owl_printdefs
 			owl_readconfig
 			owl_running
@@ -65,14 +70,14 @@ our $DEF_CONFIG     = "owl.conf";			# Default config file.
 # Constants for configuration data.
 #
 
-my $DEF_INTERVAL = 60;		# Default interval between transfers.
-my $MIN_INTERVAL = 60;		# Minimum interval between transfers.
+our $DEF_INTERVAL = 60;		# Default interval between transfers.
+our $MIN_INTERVAL = 60;		# Minimum interval between transfers.
 
-my $MININT	= 10;		# Minimum query interval.
-my $DEF_QUINT	= 60;		# Default query interval.
+our $MININT	= 10;		# Minimum query interval.
+our $DEF_QUINT	= 60;		# Default query interval.
 				# Default query timeout is half query interval.
 
-my $MINTO	= 5;		# Minimum query timeout.
+our $MINTO	= 5;		# Minimum query timeout.
 
 #
 # Amount of time between rolling the data files.
@@ -82,15 +87,15 @@ my $MINTO	= 5;		# Minimum query timeout.
 # my $DEF_ROLLINT = 60 * 20;		# Roll once every 20 minutes.
 # my $DEF_ROLLINT = 60 * 15;		# Roll once every 15 minutes.
 
-my $DEF_ROLLINT = 60 * 30;	#  XXX - testing
+our $DEF_ROLLINT = 60 * 30;	#  XXX - testing
 
-my $MIN_ROLLINT = 60 * 10;		#  Minimum file rollover interval.
+our $MIN_ROLLINT = 60 * 10;		#  Minimum file rollover interval.
 
 #
 # The default and valid DNS queries.
 #
-my $DEF_QUERYTYPE = 'A';
-my %dnsqueries =
+my $DEF_DNSTIMER_QUERY = 'A';
+my %dnstimer_queries =
 (
 	'A'		=> 1,
 	'AAAA'		=> 1,
@@ -115,18 +120,62 @@ my %dnsqueries =
 my $progname;					# Program we're part of.
 my $owllog;					# The log object.
 
-my $OWL_SENSOR	= 1;				# Sensor daemon.
-my $OWL_MANAGER	= 1;				# Sensor manager.
+our $OWL_SENSOR	 = 1;				# Owl sensor daemon.
+our $OWL_MANAGER = 2;				# Owl manager.
 
-my %owldaemons =				# Owl daemons.
+#
+# Owl program/Owl host mapping.
+# The hash key is the Owl program.
+# The hash value is a flag indicating if it runs on the sensor or manager.
+#
+our %owldaemons =				# Owl daemons.
 (
 	'owl-dnstimer'		=> $OWL_SENSOR,
+	'owl-resources'		=> $OWL_SENSOR,
+	'owl-rrdata'		=> $OWL_SENSOR,
+	'owl-rrsec' 		=> $OWL_SENSOR,
 	'owl-sensord'		=> $OWL_SENSOR,
 	'owl-transfer'		=> $OWL_SENSOR,
 
 	'owl-transfer-mgr'	=> $OWL_MANAGER,
 );
 
+#
+# Valid Owl queries.
+# The hash key is the Owl-query in the config file on the "query" lines.
+#     The keys will be used the values in @cf_owlqueries.
+# The hash value is the actual program that runs for that query.
+#
+our %owlqueries =
+(
+	'dnstimer'	=> 'owl-dnstimer',
+	'rrdata'	=> 'owl-rrdata',
+	'rrsec'		=> 'owl-rrsec',
+);
+
+#
+# The default argument for each Owl query.
+#
+my %DEF_QUERYARG =
+(
+	'dnstimer'	=> $DEF_DNSTIMER_QUERY,
+	'rrdata'	=> '',
+	'rrsec'		=> '',
+);
+
+#
+# Owl data file extensions.
+# The hash key is a file extension for a data file.
+# The hash value is the program that uses that file extension.
+#
+our %owlexts =
+(
+	'dns'		=> 'owl-dnstimer',
+	'rrec'		=> 'owl-rrdata',
+	'rrsec'		=> 'owl-rrsec',
+	'rsrc'		=> 'owl-resources',
+);
+
 #------------------------------------------------------------------------
 # Shared data.
 #
@@ -150,6 +199,9 @@ our @sshusers = ();			# Users on remote hosts for data.
 #
 our $dnstimerargs;			# Arguments for the owl-dnstimer daemon.
 our $transferargs;			# Arguments for owl-transfer daemon.
+our $resourcesargs;			# Arguments for owl-resources daemon.
+our $rrdataargs;			# Arguments for owl-rrdata daemon.
+our $rrsecargs;				# Arguments for owl-rrsec daemon.
 our $transfermgrargs;			# Arguments for owl-transfer-mgr daemon.
 
 #
@@ -158,6 +210,7 @@ our $transfermgrargs;			# Arguments for owl-transfer-mgr daemon.
 our $transfermgrargs;			# Arguments for owl-transfer-mgr daemon.
 
 our $admins = 'root';			# Administrator email.
+our $executors;				# Owl programs to execute.
 our $hesitation;			# Sleep time between executions.
 our $hibernation;			# Sleep time for minion xtn problems.
 our $quickcount;			# Consecutive quick xtns before pausing.
@@ -167,18 +220,21 @@ our $quickseconds;			# Seconds that make a quick execution.
 # Exported fields from the Owl configuration file.
 #
 
-our @cf_targets	  = ();				# List of targets.
-our @cf_servers	  = ();				# List of nameservers.
-our @cf_qtypes	  = ();				# List of DNS query types.
-our @cf_intervals = ();				# List of query intervals.
-our @cf_timeouts  = ();				# List of query timeouts.
-our @cf_rollints  = ();				# Datafile rollover interval.
-our @cf_states	  = ();				# State of targets.
+our @cf_owlqueries = ();			# List of Owl queries.
+our @cf_targets	   = ();			# List of targets.
+our @cf_servers	   = ();			# List of nameservers.
+our @cf_qargs	   = ();			# List of query arguments.
+our @cf_intervals  = ();			# List of query intervals.
+our @cf_timeouts   = ();			# List of query timeouts.
+our @cf_rollints   = ();			# Datafile rollover interval.
+our @cf_states	   = ();			# State of targets.
 
 #----------------------------------------------------
 
 our %loginfo = ();			# Logging information.
 
+our $logflag = 1;			# Log flag.
+
 #===============================================================================
 #
 # Exported routines.
@@ -321,7 +377,7 @@ sub owl_printdefs
 	print "default query interval                $DEF_QUINT\n";
 	print "minimum query interval                $MININT\n";
 	print "minimum query timeout                 $MINTO\n";
-	print "default query type                    $DEF_QUERYTYPE\n";
+	print "default query type                    $DEF_DNSTIMER_QUERY\n";
 	print "default datafile rollover interval    $DEF_ROLLINT\n";
 	print "minimum datafile rollover interval    $MIN_ROLLINT\n";
 	print "default interval between transfers    $DEF_INTERVAL\n";
@@ -481,6 +537,50 @@ sub owl_readconfig
 
 	}
 
+	#
+	# Ensure our execution targets are all valid.  If none were given,
+	# then we'll use the default values.
+	#
+	if($executors ne '')
+	{
+		my @badactors = ();		# Invalid execution targets.
+
+		#
+		# Look at all the given execution targets.  Any that we
+		# don't recognize, we'll add to a list.
+		#
+		foreach my $xqtr (split /\s/, $executors)
+		{
+			if(!defined($owldaemons{lc($xqtr)}))
+			{
+				push @badactors, $xqtr;
+			}
+		}
+
+		#
+		# If we found any unrecognized exeuction targets, give
+		# an error message.
+		#
+		if(@badactors > 0)
+		{
+			print STDERR "invalid execution targets:  " . join(', ', @badactors) . "\n";
+			$errs++;
+		}
+
+		#
+		# Make sure everything is lowercase.
+		#
+		$executors = lc($executors);
+	}
+	else
+	{
+		#
+		# If nothing was specified, we'll specify the default and
+		# let our caller decide how to interpret that.
+		#
+		$executors = 'default';
+	}
+
 	#
 	# Command-line values take precedence over config-file values.
 	#
@@ -528,6 +628,11 @@ sub owl_setlog
 	my @chron;				# Current times.
 	my $tstmp;				# Timestamp for logname.
 
+	#
+	# If logging is turned off, we won't build a log file.
+	#
+	return(undef) if(! $logflag);
+
 	#
 	# Get the name of the host we're sitting on.
 	#
@@ -572,7 +677,19 @@ sub owl_setlog
 	#
 	# Build the name of the log file we'll be using.
 	#
-	$loginfo{filename} = File::Spec->catfile($loginfo{logdir},$loginfo{logfile});
+#	$loginfo{filename} = File::Spec->catfile($loginfo{logdir},$loginfo{logfile});
+
+	#
+	# Make sure the log will rotate periodically.
+	#
+#	$owllog->add(Log::Dispatch::FileRotate->new(
+#					name      => "$pname logfile",
+#					min_level => 'debug',
+#					mode      => 'append',
+#					filename  => "$loginfo{filename}",
+#					size      => 2000000,
+#					max	  => 60)
+#		       );
 
 	#
 	# Make sure the log will rotate periodically.
@@ -581,7 +698,7 @@ sub owl_setlog
 					name      => "$pname logfile",
 					min_level => 'debug',
 					mode      => 'append',
-					filename  => "$loginfo{filename}",
+					filename  => File::Spec->catfile($loginfo{logdir},$loginfo{logfile}),
 					size      => 2000000,
 					max	  => 60)
 		       );
@@ -592,6 +709,30 @@ sub owl_setlog
 	return($owllog);
 }
 
+#----------------------------------------------------------------------
+# Routine:      owl_log()
+#
+# Purpose:      Give the appropriate type of log message -- unless we
+#		aren't supposed to do any logging.
+#
+sub owl_log
+{
+	my $sensorlog	= shift;		# Log object.
+	my $mtype	= shift;		# Type of message to log.
+	my $msg		= shift;		# Message to log.
+
+	return if($logflag == 0);
+
+	if($mtype = 0)
+	{
+		$sensorlog->log($msg);
+	}
+	else
+	{
+		$sensorlog->warning("$msg");
+	}
+}
+
 #------------------------------------------------------------------------
 # Routine:	owl_singleton()
 #
@@ -816,6 +957,18 @@ sub conf_hostline
 	{
 		$dnstimerargs = join(' ', @atoms);
 	}
+	elsif($keyword =~ /^resources-args$/i)
+	{
+		$resourcesargs = join(' ', @atoms);
+	}
+	elsif($keyword =~ /^rrdata-args$/i)
+	{
+		$rrdataargs = join(' ', @atoms);
+	}
+	elsif($keyword =~ /^rrsec-args$/i)
+	{
+		$rrsecargs = join(' ', @atoms);
+	}
 	elsif($keyword =~ /^transfer-args$/i)
 	{
 		$transferargs = join(' ', @atoms);
@@ -828,6 +981,10 @@ sub conf_hostline
 	{
 		$admins = join(' ', @atoms);
 	}
+	elsif($keyword =~ /^execute$/i)
+	{
+		$executors = join(' ', @atoms);
+	}
 	elsif($keyword =~ /^hesitation$/i)
 	{
 		$hesitation = $atoms[0];
@@ -903,20 +1060,22 @@ sub conf_logline
 #
 #		Type target config values:
 #			0 - "query" (keyword)
-#			1 - hostname of target
-#			2 - nameserver to query for target
-#			3 - type of query (optional)
-#			4 - keyword (optional)
-#			5 - value (optional)
+#			1 - type of Owl query
+#			2 - hostname of target
+#			3 - nameserver to query for target
+#			4 - query argument (optional)
+#			5 - keyword (optional)
+#			6 - value (optional)
 #
 sub conf_queryline
 {
 	my @atoms = @_;			# Components of the log config line.
 	my $atoms = @atoms;		# Count of line's atoms.
 
+	my $owlquery;			# Type of Owl query.
 	my $target;			# Target name.
 	my $nsname;			# Nameserver's name.
-	my $qtype;			# Query type.
+	my $qarg;			# Query argument.
 	my $interval;			# Entry's query interval.
 	my $timeout;			# Entry's query timeout.
 	my $rollint;			# Datafile rollover interval.
@@ -924,21 +1083,26 @@ sub conf_queryline
 	#
 	# Ensure we've got at least target and nameserver names for this entry.
 	#
-	if($atoms < 2)
+	if($atoms < 4)
 	{
-		print STDERR "owl_readconfig:  no target or nameserver given for \"target\" line\n";
+		print STDERR "owl_readconfig:  each query line MUST have an Owl query, a target, and a nameserver; \"target\"\n";
 		return(1);
 	}
-	elsif($atoms < 3)
+
+	#
+	# Get the Owl query type and ensure it's okay.
+	#
+	shift @atoms;
+	$owlquery = shift @atoms;
+	if(!defined($owlqueries{$owlquery}))
 	{
-		print STDERR "owl_readconfig:  no nameserver name given for \"target\" line\n";
+		print STDERR "owl_readconfig:  invalid Owl query type \"$owlquery\"\n";
 		return(1);
 	}
 
 	#
 	# Get the target.
 	#
-	shift @atoms;
 	$target = shift @atoms;
 
 	#
@@ -948,16 +1112,20 @@ sub conf_queryline
 	$nsname = "$nsname.$NSBASE" if($nsname =~ /^[a-m]$/i);
 
 	#
-	# Get the query type and ensure it's okay.
+	# Get the query argument and ensure it's okay.
 	#
 	if(@atoms > 0)
 	{
-		$qtype = shift @atoms;
-		$qtype = uc($qtype);
-		if(! defined($dnsqueries{$qtype}))
+		$qarg = shift @atoms;
+		$qarg = uc($qarg);
+
+		if($owlquery eq 'dnstimer')
 		{
-			print STDERR "owl_readconfig:  invalid query type \"$qtype\" for $target:$nsname target\n";
-			return(1);
+			if(! defined($dnstimer_queries{$qarg}))
+			{
+				print STDERR "owl_readconfig:  invalid query type \"$qarg\" for dnstimer $target:$nsname target\n";
+				return(1);
+			}
 		}
 	}
 
@@ -969,11 +1137,11 @@ sub conf_queryline
 	$rollint  = shift @atoms if(@atoms > 0);
 
 	#
-	# Account for default query type and missing query type.
+	# Account for default query argument and missing query argument.
 	#
-	if(($qtype eq '') || ($qtype eq '-'))
+	if(($qarg eq '') || ($qarg eq '-'))
 	{
-		$qtype = $DEF_QUERYTYPE;
+		$qarg = $DEF_QUERYARG{$owlquery};
 	}
 
 	#
@@ -1007,9 +1175,10 @@ sub conf_queryline
 	#
 	# Save the data for this entry.
 	#
+	push @cf_owlqueries, $owlquery;
 	push @cf_targets, $target;
 	push @cf_servers, $nsname;
-	push @cf_qtypes,  $qtype;
+	push @cf_qargs,  $qarg;
 	push @cf_intervals, $interval;
 	push @cf_timeouts,  $timeout;
 	push @cf_rollints,  $rollint;
@@ -1148,7 +1317,8 @@ owlutils - Utility routines for Owl programs.
 
   owl_chkdir('archive','oldfiles');
 
-  owl_setlog('owl-dnstimer');
+  $sensorlog = owl_setlog('owl-dnstimer');
+  owl_log($sensorlog,0,'owl-dnstimer has started')>
 
   owl_getpid();
 
@@ -1227,15 +1397,16 @@ The associated arrays of Owl configuration data hold the contents of the
 I<query> lines from the configuration file.  They are associated in that
 a particular I<query> line has its data distributed across the arrays all
 at the same index.  Therefore, I<$cf_servers[4]> holds a datum for the same
-I<query> line as I<$cf_targets[4]> and I<$cf_qtypes[4]>.  If a I<query>
+I<query> line as I<$cf_targets[4]> and I<$cf_qargs[4]>.  If a I<query>
 line does not contain data fields for each array, then a default value
 will be used.  
 
 Data set from a I<query> line:
 
+    @cf_owlqueries           List of Owl queries.
     @cf_targets	             List of targets.
     @cf_servers	             List of nameservers.
-    @cf_qtypes	             List of DNS query types.
+    @cf_qargs	             List of DNS query types.
     @cf_intervals            List of query intervals.
     @cf_timeouts             List of query timeouts.
     @cf_rollints             Datafile rollover interval.
@@ -1256,6 +1427,7 @@ Data specified by calling programs or built by B<owlutils.pm>:
     $conffile                Configuration file.
     $datadir                 Data directory.
     $logdir                  Log directory.
+    $logflag                 Log flag -- 0 is nologging; 1 - is logging.
     $pidfile                 Filename of process-id file.
 
     $hesitation              Sleep time between executions.
@@ -1267,9 +1439,15 @@ Data specified on a I<host> line:
 
     $admins                  Administrator email addresses.
     $dnstimerargs            Arguments for the owl-dnstimer daemon.
+    $executors               Programs to execute.  This may be
+                             "default", "dnstimer", "owl-dnstimer",
+			     "transfer", or "owl-transfer".
     $hostname                Name of this host.
-    $transferargs            Arguments for owl-transfer daemon.
-    $transfermgrargs         Arguments for owl-transfer-mgr daemon.
+    $resourcesargs           Arguments for the owl-resources daemon.
+    $rrdataargs              Arguments for the owl-rrdata daemon.
+    $rrsecargs               Arguments for the owl-rrsec daemon.
+    $transferargs            Arguments for the owl-transfer daemon.
+    $transfermgrargs         Arguments for the owl-transfer-mgr daemon.
 
 Data specified on a I<data> line:
 
@@ -1329,6 +1507,18 @@ Return Values:
         if a non-Owl program is named.
     >0 - Program halted; the count of halted programs is returned.
 
+=item I<owl_log(sensorlog,msgtype,msg)>
+
+This routine adds an entry to the log file specified by I<sensorlog>.
+I<sensorlog> is an object previously returned by I<owl_setlog()>.  If
+the I<$logflag> is set to 0, then a log message will not be written.
+
+The I<msgtype> parameter specifies the type of log message that will be
+written.  If it is 0, then a regular log message will be given.  If it
+is 1, then a warning message will be given.
+
+The I<msg> parameter is the message to be written to the log file.
+
 =item I<owl_printdefs()>
 
 This routine prints the default values used by this module, then it exits.
@@ -1350,8 +1540,8 @@ Data from "host" lines:
 
     $owlutils::admins          - email addresses for Owl
 				 administrators
-    $owlutils::dnstimerargs    - arguments for the owl-dnstimer
-				 daemon
+    $owlutils::dnstimerargs    - arguments for the owl-dnstimer daemon
+    $owlutils::executors       - programs to be executed
     $owlutils::hesitation      - time between Owl daemon executions
     $owlutils::hibernation     - sleep time upon execution problems
     $owlutils::quickcount      - count of quick executions before
@@ -1359,10 +1549,11 @@ Data from "host" lines:
     $owlutils::quickseconds    - seconds count that makes a quick
 			         execution
     $owlutils::hostname        - name of this host
-    $owlutils::transferargs    - arguments for the owl-transfer
-				 daemon
-    $owlutils::transfermgrargs - arguments for the owl-transfer-mgr
-				 daemon
+    $owlutils::resourcesargs   - arguments for the owl-resources daemon
+    $owlutils::rrdataargs      - arguments for the owl-rrdata daemon
+    $owlutils::rrsecargs       - arguments for the owl-rrsec daemon
+    $owlutils::transferargs    - arguments for the owl-transfer daemon
+    $owlutils::transfermgrargs - arguments for the owl-transfer-mgr daemon
 
 Data from "data" lines:
 
@@ -1376,13 +1567,14 @@ Data from "log" lines:
 
 Data from "query" lines:
 
-    @owlutils::cf_intervals - query intervals from "query" lines
-    @owlutils::cf_qtypes    - DNS query types from "query" lines
-    @owlutils::cf_rollints  - interval from "query" lines
-    @owlutils::cf_servers   - nameservers from "query" lines
-    @owlutils::cf_states    - targets from "query" lines
-    @owlutils::cf_targets   - targets from "query" lines
-    @owlutils::cf_timeouts  - query timeouts from "query" lines
+    @owlutils::cf_intervals  - query intervals from "query" lines
+    @owlutils::cf_owlqueries - Owl queries from "query" lines
+    @owlutils::cf_qargs      - query arguments from "query" lines
+    @owlutils::cf_rollints   - interval from "query" lines
+    @owlutils::cf_servers    - nameservers from "query" lines
+    @owlutils::cf_states     - targets from "query" lines
+    @owlutils::cf_targets    - targets from "query" lines
+    @owlutils::cf_timeouts   - query timeouts from "query" lines
 
 Data from "remote" lines:
 
@@ -1488,6 +1680,7 @@ Return Values:
 
 B<owl-dnstimer(1)>,
 B<owl-sensord(1)>,
+B<owl-rrdata(1)>,
 B<owl-transfer(1)>,
 B<owl-transfer-mgr(1)>
 
