@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright 2005-2013 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2005-2014 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details.
 #
 # DNSSEC-Tools
@@ -16,8 +16,6 @@
 
 package Net::DNS::SEC::Tools::tooloptions;
 
-use UNIVERSAL qw (can);
-
 use Net::DNS::SEC::Tools::QWPrimitives;
 use Net::DNS::SEC::Tools::defaults;
 
@@ -27,8 +25,8 @@ use strict;
 use Net::DNS::SEC::Tools::conf;
 use Net::DNS::SEC::Tools::keyrec;
 
-our $VERSION = "2.0";
-our $MODULE_VERSION = "2.0.0";
+our $VERSION = "2.1";
+our $MODULE_VERSION = "2.1.0";
 
 our @ISA = qw(Exporter);
 
@@ -836,7 +834,7 @@ sub localgetoptions
 	#
 	if(!$ret && $errexit)
 	{
-		main::usage() if(can('main','usage'));
+		main::usage() if(main->can('usage'));
 		exit(1);
 	}
 
@@ -1117,7 +1115,7 @@ The old exit action is returned.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2013 SPARTA, Inc.  All rights reserved.
+Copyright 2005-2014 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
