@@ -1,6 +1,6 @@
 #ifndef DNSSECCHECKVERSION_H
 #define DNSSECCHECKVERSION_H
 
-#define .1"
+#define DNSSEC_CHECK_VERSION "2.1"
 
 #endif // DNSSECCHECKVERSION_H
