@@ -19,6 +19,10 @@
 #include <arpa/nameser_compat.h>
 #endif
 
+#ifndef RES_USE_DNSSEC
+#define RES_USE_DNSSEC  0x00200000
+#endif
+
 /* libsres functions that they don't export */
 extern "C" {
     struct expected_arrival *
@@ -28,6 +32,7 @@ extern "C" {
     void    res_switch_all_to_tcp(struct expected_arrival *ea);
     int     res_io_queue_ea(int *transaction_id, struct expected_arrival *new_ea);
     int     res_io_send(struct expected_arrival *shipit);
+    int     res_sq_free_expected_arrival(struct expected_arrival **ea);
 }
 
 /* Syncronous macros */
@@ -131,6 +136,18 @@ async_requests_remaining() {
     return outstandingCount;
 }
 
+void
+async_cancel_outstanding() {
+    int i;
+    for(i = 0; i < maxcount; i++) {
+        if (outstanding_queries[i].live) {
+            res_io_cancel_all_remaining_attempts(outstanding_queries[i].ea);
+            outstanding_queries[i].live = 0;
+        }
+    }
+    outstandingCount = 0;
+}
+
 void
 check_queued_sends() {
     int i;
@@ -186,6 +203,7 @@ check_outstanding_async() {
         outstandingCount--;
 
         outstanding_queries[i].live = 0;
+        res_sq_free_expected_arrival(&outstanding_queries[i].ea);
     }
 }
 
@@ -223,11 +241,14 @@ collect_async_query_select_info(fd_set *udp_fds, int *numUdpFds, fd_set *tcp_fds
     tv.tv_sec = 0;
     tv.tv_usec = 1;
 
+    int live = 0;
     for(i = 0; i < maxcount; i++) {
-        if (outstanding_queries[i].ea->ea_using_stream)
+        if (outstanding_queries[i].ea && outstanding_queries[i].ea->ea_using_stream)
             res_async_query_select_info(outstanding_queries[i].ea, numTcpFds, tcp_fds, &tv);
         else
             res_async_query_select_info(outstanding_queries[i].ea, numUdpFds, udp_fds, &tv);
+        if (outstanding_queries[i].live)
+            live++;
     }
 }
 
@@ -302,13 +323,11 @@ static struct name_server *
 _parse_name_server(char *ns_name, int flags) {
     struct name_server *ns;
 
-    ns = parse_name_server(ns_name, NULL);
+    ns = parse_name_server(ns_name, NULL, flags);
 
     if (!ns)
         return NULL;
 
-    ns->ns_options |= flags;
-
     /* 1 retry at 1 second */
     ns->ns_retrans = 1;
     ns->ns_retry = 1;
@@ -507,11 +526,19 @@ int check_small_edns0_results(u_char *response, size_t response_size, char *buf,
     if (!found_edns0)
         RETURN_ERROR("No EDNS0 record was found in the response but one was expected.");
 
+    /* per discussion with many folk (especially Olafur), this test is invalid as
+     * EDNS0 indicates what they accept, and not what they can send.  So google (for example)
+     * has a low EDNS0 and that's perfectly valid because you can still request larger things from
+     * them and there is no promise that they won't deliver.  You actually can't tell what they will
+     * do because that's not negotated; you can only tell what they'll accept.
+     */
+#ifdef EDNS0_SIZE_NO_LONGER_VALID
     if (found_edns0 < 1480) {
         snprintf(buf, buf_len, "Warning: The returned EDNS0 size (%d) is smaller than recommended (1480)", found_edns0);
         *testStatus = CHECK_WARNING;
         return CHECK_WARNING;
     }
+#endif
 
     *testStatus = CHECK_SUCCEEDED;
     snprintf(buf, buf_len, "Success: The returned EDNS0 size (%d) was reasonable.", found_edns0);
@@ -935,4 +962,68 @@ int check_can_get_ds_async(char *ns_name, char *buf, size_t buf_len, int *testSt
 }
 
 
+#endif /* VAL_NO_ASYNC */
+
+int check_can_get_signed_dname(char *ns_name, char *buf, size_t buf_len, const char *name, int rrtype, const char *rrtypename) {
+    // XXX
+}
+
+
+#ifndef VAL_NO_ASYNC
+void _check_dname_async_response(u_char *response, size_t response_size,
+                                 int rc, int *testStatus, char *buf, size_t buf_len, void *localData) {
+    int             rrnum;
+    ns_msg          handle;
+    ns_rr           rr;
+
+    if (rc != SR_UNSET)
+        SET_ERROR("Query for a dname response failed to get an answer entirely");
+
+
+    /* Check for DNAME */
+    rrnum = count_types(response, response_size, buf, buf_len, ns_t_dname, ns_s_an, "DNAME");
+
+    if (rrnum <= 0)
+        SET_ERROR("Failed to find an expected DNAME record.");
+
+
+
+    /* Check for an RRSIG that signs the DNAME */
+    if (ns_initparse(response, response_size, &handle) < 0)
+        SET_ERROR("Fatal internal error: failed to init parser");
+    rrnum = 0;
+    for (;;) {
+        if (ns_parserr(&handle, ns_s_an, rrnum, &rr)) {
+            if (errno != ENODEV) {
+                /* parse error */
+                SET_ERROR("failed to parse a returned additional RRSET");
+            }
+            break; /* out of data */
+        }
+        if (ns_rr_type(rr) == ns_t_rrsig) {
+            // XXX: check signing type
+            SET_SUCCESS("Querying for a record in a DNAMEd zone returned a DNAME and a signature.");
+        }
+        rrnum++;
+    }
+
+    SET_ERROR("Failed to find an expected RRSIG on the DNAME record.");
+}
+
+int check_can_get_signed_dname_async(char *ns_name, char *buf, size_t buf_len, int *testStatus) {
+    /* queries with the DO bit for something that should be DNAMEd to elsewhere.  The answer should have both a DNAME record,
+     * and a signature on that record. The CNAME is optional so we don't check it.
+     */
+
+    struct name_server *ns;
+    struct expected_arrival *ea;
+
+    ns = _parse_name_server(ns_name, SR_QUERY_VALIDATING_STUB_FLAGS | SR_QUERY_RECURSE);
+
+    ea = res_async_query_send("good-a.dname-good-ns.test.dnssec-tools.org", ns_t_a, ns_c_in, ns);
+    add_outstanding_async_query(ea, _check_dname_async_response,
+                                testStatus, buf, buf_len, NULL);
+    return CHECK_CRITICAL;
+}
+
 #endif /* VAL_NO_ASYNC */
