@@ -38,6 +38,8 @@
 **
 ****************************************************************************/
 
+#include <QMessageBox>
+
 #include "graphwidget.h"
 #include "edge.h"
 #include "node.h"
@@ -289,8 +291,6 @@ void GraphWidget::wheelEvent(QWheelEvent *event)
 
 void GraphWidget::drawBackground(QPainter *painter, const QRectF &rect)
 {
-    Q_UNUSED(rect);
-
     // Shadow
     QRectF sceneRect = QRectF(mapToScene(0, 0), mapToScene(width(), height())); //scene()->sceneRect();
 
@@ -298,7 +298,7 @@ void GraphWidget::drawBackground(QPainter *painter, const QRectF &rect)
     QLinearGradient gradient(sceneRect.topLeft(), sceneRect.bottomRight());
     gradient.setColorAt(0, Qt::white);
     gradient.setColorAt(1, Qt::lightGray);
-    painter->fillRect(rect.intersect(sceneRect), gradient);
+    painter->fillRect(rect.intersected(sceneRect), gradient);
     painter->setBrush(Qt::NoBrush);
     painter->drawRect(sceneRect);
 }
@@ -759,7 +759,7 @@ void GraphWidget::about()
     msgBox.setWindowTitle("About DNSSEC-Nodes");
     msgBox.setText("<img src=\":/icons/dnssec-nodes-64x64.png\" style=\"float: right\"/>"
                    "<h1>DNSSEC-Nodes</h1><p><i>A component of the DNSSEC-Tools Project</i></p><p>For more information, please visit http://www.dnssec-tools.org/.</i></p>"
-                   "<p>DNSSEC-Tools Version: 2.0</p>");
+                   "<p>DNSSEC-Tools Version: 2.1</p>");
     msgBox.setStandardButtons(QMessageBox::Close);
     msgBox.exec();
 }
