@@ -14,7 +14,6 @@
  * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
  * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #define NS_TYPE_ELT			0x40    /* EDNS0 extended label type */
@@ -182,7 +181,7 @@ ns_name_pton(const char *src, u_char * dst, size_t dstsiz)
 {
     u_char         *label, *bp, *eom;
     int             c, n, escaped, e = 0;
-    char           *cp;
+    const char     *cp;
 
     escaped = 0;
     bp = dst;
