@@ -57,7 +57,11 @@ static int      addstr(const char *src, size_t len,
                        char **buf, size_t * buflen);
 static int      addtab(size_t len, size_t target, int spaced,
                        char **buf, size_t * buflen);
-
+int             ns_sprintrrf_data(const u_char * msg, size_t msglen,
+                  const char *name, ns_class class_h, ns_type type_h,
+                  u_long ttl, const u_char * rdata, size_t rdlen,
+                  const char *origin,
+                  char *buf, size_t buflen);
 /*
  * Macros. 
  */
@@ -133,16 +137,9 @@ ns_sprintrrf(const u_char * msg, size_t msglen,
              const char *name_ctx, const char *origin,
              char *buf, size_t buflen)
 {
-    const char *addr = NULL;
     const char     *obuf = buf;
-    const u_char   *edata = rdata + rdlen;
     int             spaced = 0;
-    struct sockaddr_in sa;
-#ifdef VAL_IPV6
-    struct sockaddr_in6 sa6;
-#endif
 
-    const char     *comment;
     char            tmp[100];
     int             len, x;
 
@@ -182,7 +179,36 @@ ns_sprintrrf(const u_char * msg, size_t msglen,
     T(addstr(tmp, len, &buf, &buflen));
     if (rdlen == 0U)
         return (buf - obuf);
-    T(spaced = addtab(x + len, 16, spaced, &buf, &buflen));
+     T(spaced = addtab(x + len, 16, spaced, &buf, &buflen));
+
+    return ns_sprintrrf_data(msg, msglen, name, class_h, type_h,
+                             ttl, rdata, rdlen, origin,
+                             buf, buflen);
+}
+
+int
+ns_sprintrrf_data(const u_char * msg, size_t msglen,
+                  const char *name, ns_class class_h, ns_type type_h,
+                  u_long ttl, const u_char * rdata, size_t rdlen,
+                  const char *origin,
+                  char *buf, size_t buflen)
+{
+    struct sockaddr_in sa;
+#ifdef VAL_IPV6
+    struct sockaddr_in6 sa6;
+#endif
+    const char *addr = NULL;
+    int len;
+    const u_char   *edata = rdata + rdlen;
+    int             spaced = 0;
+    char            tmp[100];
+    const char     *obuf = buf;
+    const char     *comment;
+
+    memset(&sa, 0, sizeof(sa));
+#ifdef VAL_IPV6
+    memset(&sa6, 0, sizeof(sa6));
+#endif
 
     /*
      * RData.
@@ -494,12 +520,11 @@ ns_sprintrrf(const u_char * msg, size_t msglen,
         }
 
     case ns_t_ds:{
-            u_int           key_id;
             u_int           algo;
             u_int           digest_type;
             u_int           hashlen = 0;
 
-            RES_GET16(key_id, rdata);
+            rdata += NS_INT16SZ; /* skip key_id */
             algo = *rdata++ & 0xF;
             digest_type = *rdata++ & 0xF;
 
