@@ -171,7 +171,6 @@ do_section(ns_msg * handle, ns_sect section, int pflag, FILE * file)
     int             n, rrnum;
     const static int buflen = 2048;
     char           *buf;
-    ns_opcode       opcode;
     ns_rr           rr;
 #ifdef HAVE_STRERROR_R
     char            err_buf[ERRBUFLEN + 1];
@@ -183,7 +182,7 @@ do_section(ns_msg * handle, ns_sect section, int pflag, FILE * file)
         return;
     }
 
-    opcode = (ns_opcode) libsres_msg_getflag(*handle, ns_f_opcode);
+    (void) libsres_msg_getflag(*handle, ns_f_opcode);
     rrnum = 0;
     for (;;) {
         if (ns_parserr(handle, section, rrnum, &rr)) {
@@ -447,6 +446,7 @@ const struct RES_SYM_TYPE __p_type_sres_syms[] = {
     {ns_t_dlv, "DLV", "DLV record"},
 #endif
     {ns_t_ds, "DS", "delegation signer"},
+    {ns_t_tlsa, "TLSA", "DANE TLSA"},
     {0, NULL, NULL}
 };
 
@@ -473,8 +473,13 @@ const struct RES_SYM_TYPE __p_rcode_syms[] = {
     {0, NULL, NULL}
 };
 
+#if defined(__OpenBSD__)
+int
+sym_ston(const struct RES_SYM_TYPE *syms, char *name, int *success)
+#else
 int
 sym_ston(const struct RES_SYM_TYPE *syms, const char *name, int *success)
+#endif
 {
     size_t namelen = 0;
     if (name)
@@ -983,7 +988,7 @@ loc_ntoa(const u_char *binary, char *ascii)
 
     const u_int32_t referencealt = 100000 * 100;
 
-    u_int32_t       latval, longval, altval;
+    int             latval, longval, altval;
     u_int32_t       templ;
     u_char          sizeval, hpval, vpval, versionval;
 
@@ -1077,8 +1082,13 @@ loc_ntoa(const u_char *binary, char *ascii)
 /*
  * Return the number of DNS hierarchy levels in the name. 
  */
+#if defined(__OpenBSD__)
+int
+dn_count_labels(char *name)
+#else
 int
 dn_count_labels(const char *name)
+#endif
 {
     int             i, len, count;
 
