@@ -16,7 +16,7 @@
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
 /*
- * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2013 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #include "validator-internal.h"
@@ -43,23 +43,20 @@ wire_name_length(const u_char * field)
     if (field == NULL)
         return 0;
 
-    for (j = 0; field[j] && !(0xc0 & field[j]) && j < NS_MAXCDNAME;
+    /* 
+     * Check that the total length is less than max allowable, and that
+     * label length does not exceed 63
+     */
+    for (j = 0; j < NS_MAXCDNAME && field[j] && !(0xc0 & field[j]);
          j += field[j] + 1);
-    if (field[j])
-        j++;
-    j++;
 
-    if (j > NS_MAXCDNAME)
+    /* 
+     * return 0 if we exceed max allowable label length or total length
+     */
+    if (j >= NS_MAXCDNAME || field[j])
         return 0;
     else
-        return j;
-}
-
-
-static size_t
-skip_questions(const u_char * buf)
-{
-    return 12 + wire_name_length(&buf[12]) + 4;
+        return j+1;
 }
 
 void
@@ -96,58 +93,15 @@ dump_response(const u_char * ans, size_t resplen)
         } while (k < resplen);
 }
 
-
-u_int16_t
-retrieve_type(const u_char * rr)
-{
-    u_int16_t       type_n;
-    size_t          name_length = wire_name_length(rr);
-
-    memcpy(&type_n, &rr[name_length], sizeof(u_int16_t));
-    return ntohs(type_n);
-}
-
 int
 res_quecmp(u_char * query, u_char * response)
 {
-    size_t length;
-
     if (query == NULL || response == NULL)
         return 1;
 
     return namecmp(&query[12], &response[12]);
 }
 
-int
-right_sized(u_char * response, size_t response_length)
-{
-    HEADER         *header = (HEADER *) response;
-    size_t         index = skip_questions(response);
-    size_t         records =
-        ntohs(header->ancount) + ntohs(header->nscount)
-        + ntohs(header->arcount);
-    size_t  i;
-    u_int16_t  rdata_len_n;
-
-    if (index > response_length)
-        return TRUE;
-
-    for (i = 0; i < records; i++) {
-        index += wire_name_length(&response[index]) + ENVELOPE;
-
-        if (index > response_length)
-            return TRUE;
-
-        memcpy(&rdata_len_n, &response[index - 2], sizeof(u_int16_t));
-        index += ntohs(rdata_len_n);
-
-        if (index > response_length)
-            return TRUE;
-    }
-
-    return index == response_length;
-}
-
 int
 theres_something_wrong_with_header(u_char * response,
                                    size_t response_length)
@@ -162,14 +116,6 @@ theres_something_wrong_with_header(u_char * response,
         return SR_HEADER_ERROR;
     }
 
-    /*
-     * Check the length and count of the records 
-     */
-    if (right_sized(response, response_length) == FALSE) {
-        res_log(NULL, LOG_DEBUG, "libsres: ""header: not right sized!");
-        return SR_HEADER_ERROR;
-    }
-
     /*
      * Check the RCODE value.
      * RCODE of no error is always welcome 
@@ -180,46 +126,13 @@ theres_something_wrong_with_header(u_char * response,
     /*
      * RCODE of NXDOMAIN (no such domain) is welcome in some circumtances:
      * With no other records present
-     * With an SOA or NXT in the authority (ns) section
+     * With records in the authority (ns) section
      */
     if (header->rcode == ns_r_nxdomain) {
-        if (header->ancount == 0 && header->nscount == 0
-            && header->arcount == 0)
+        if ((header->ancount == 0 && header->nscount == 0
+            && header->arcount == 0) || ntohs(header->nscount) > 0)
             return SR_UNSET;
 
-        /** if (ntohs(header->nscount) > 1) */
-        {
-            size_t          i;
-            size_t          auth_index = skip_questions(response);
-            u_int16_t       type_h;
-            u_int16_t       rdata_len_n;
-
-            for (i = 0; i < ntohs(header->ancount); i++) {
-                auth_index +=
-                    wire_name_length(&response[auth_index]) + ENVELOPE;
-                memcpy(&rdata_len_n, &response[auth_index - 2],
-                       sizeof(u_int16_t));
-                auth_index += ntohs(rdata_len_n);
-            }
-
-            for (i = 0; i < ntohs(header->nscount); i++) {
-                type_h = retrieve_type(&response[auth_index]);
-
-                if (type_h == ns_t_soa || 
-#ifdef LIBVAL_NSEC3
-                    type_h == ns_t_nsec3 || 
-#endif
-                    type_h == ns_t_nsec)
-                    return SR_UNSET;
-
-                auth_index +=
-                    wire_name_length(&response[auth_index]) + ENVELOPE;
-                memcpy(&rdata_len_n, &response[auth_index - 2],
-                       sizeof(u_int16_t));
-                auth_index += ntohs(rdata_len_n);
-            }
-        }
-
         res_log(NULL, LOG_DEBUG, "libsres: ""header: nxdomain!");
         return SR_NXDOMAIN;
     }
@@ -277,7 +190,7 @@ clone_ns(struct name_server **cloned_ns, struct name_server *ns)
     /*
      * Initialize the rest of the fields 
      */
-    (*cloned_ns)->ns_tsig = NULL;       //XXX Still not doing anything with TSIG
+    (*cloned_ns)->ns_tsig = clone_ns_tsig(ns->ns_tsig);
     (*cloned_ns)->ns_security_options = ns->ns_security_options;
     (*cloned_ns)->ns_status = ns->ns_status;
 
@@ -302,6 +215,9 @@ clone_ns(struct name_server **cloned_ns, struct name_server *ns)
             }
             FREE((*cloned_ns)->ns_address);
             (*cloned_ns)->ns_address = NULL;
+        } else {
+            memset((*cloned_ns)->ns_address[i], 0, 
+                    sizeof(struct sockaddr_storage));
         }
     }
 
@@ -332,8 +248,10 @@ clone_ns_list(struct name_server **ns_list,
     for (ns = orig_ns_list; ns != NULL; ns = ns->ns_next) {
 
         struct name_server *temp_ns;
-        if ((ret_val = clone_ns(&temp_ns, ns)) != SR_UNSET)
+        if ((ret_val = clone_ns(&temp_ns, ns)) != SR_UNSET) {
+            free_name_servers(ns_list);
             return ret_val;
+        }
 
         /*
          * Add the name server record to the list 
@@ -363,14 +281,17 @@ query_send(const char *name,
            int *trans_id)
 {
     int             ret_val;
-    struct timeval      dummy;
+    struct timeval  dummy;
+    struct timeval  now;
 
     ret_val = query_queue(name, type_h, class_h, pref_ns, trans_id);
     if (SR_UNSET != ret_val)
         return ret_val;
 
     timerclear(&dummy);
+    gettimeofday(&now, NULL);
     res_io_check_one_tid(*trans_id, &dummy, NULL);
+    //res_io_check_one_tid(*trans_id, &now, NULL);
 
     return SR_UNSET;
 }
