@@ -30,12 +30,6 @@
 #include "val_resquery.h"
 #include "val_cache.h"
 
-struct zone_ns_map_t {
-    u_char        zone_n[NS_MAXCDNAME];
-    struct name_server *nslist;
-    struct zone_ns_map_t *next;
-};
-
 /*
  * we have caches for DNSKEY, DS, NS/glue, answers, and proofs
  * XXX negative cache functionality is currently unimplemented
@@ -43,11 +37,6 @@ struct zone_ns_map_t {
 static struct rrset_rec *unchecked_hints = NULL;
 static struct rrset_rec *unchecked_answers = NULL;
 
-/*
- * Also maintain mapping between zone and name server, 
- */
-static struct zone_ns_map_t *zone_ns_map = NULL;
-
 #ifndef VAL_NO_THREADS
 
 /*
@@ -58,8 +47,6 @@ static pthread_rwlock_t ns_rwlock;
 static int ns_rwlock_init = 0;
 static pthread_rwlock_t ans_rwlock;
 static int ans_rwlock_init = 0;
-static pthread_rwlock_t map_rwlock;
-static int map_rwlock_init = 0;
 
 #define VAL_CACHE_LOCK_INIT(lk, initvar) \
     ((initvar != 0) || \
@@ -81,8 +68,6 @@ static int ns_rwlock = -1;
 static int ns_rwlock_init = -1;
 static int ans_rwlock = -1;
 static int ans_rwlock_init = -1;
-static int map_rwlock = -1;
-static int map_rwlock_init = -1;
 
 #define VAL_CACHE_LOCK_INIT(lk, initvar)
 #define VAL_CACHE_LOCK_SH(lk)
@@ -202,7 +187,8 @@ stow_info(struct rrset_rec **unchecked_info, struct rrset_rec **new_info, struct
 static int
 lookup_store(u_char *name_n, u_int16_t class_h, u_int16_t type_h,
              struct rrset_rec *answer_head, 
-             struct rrset_rec **new_answer)
+             struct rrset_rec **new_answer,
+             unsigned long ns_options)
 {
 
     struct rrset_rec *next_answer;
@@ -235,7 +221,13 @@ lookup_store(u_char *name_n, u_int16_t class_h, u_int16_t type_h,
                 (NULL != (u_char *) namename(name_n, 
                                     next_answer->rrs_name_n)))) {
 
-                if (next_answer->rrs_data != NULL) {
+                /* 
+                 * if we want to match particular options, make sure
+                 * they actually match
+                 */
+                if((ns_options == 0 || 
+                    ns_options == next_answer->rrs_ns_options) &&
+                   (next_answer->rrs_data != NULL)) {
                     *new_answer = copy_rrset_rec(next_answer);
                     if (*new_answer) {
                         /* Adjust the TTL */
@@ -264,6 +256,7 @@ get_cached_rrset(struct val_query_chain *matched_q,
     u_char *name_n;
     u_int16_t class_h;
     u_int16_t type_h;
+    unsigned long ns_options;
 
     int retval;
 
@@ -273,6 +266,8 @@ get_cached_rrset(struct val_query_chain *matched_q,
     name_n = matched_q->qc_name_n;
     class_h = matched_q->qc_class_h;
     type_h = matched_q->qc_type_h;
+    ns_options = (matched_q->qc_flags & VAL_QUERY_DONT_VALIDATE) ?
+        SR_QUERY_VALIDATING_STUB_FLAGS : 0;
 
     new_answer = NULL;
     *response = NULL;
@@ -281,7 +276,7 @@ get_cached_rrset(struct val_query_chain *matched_q,
     VAL_CACHE_LOCK_SH(&ans_rwlock);
 
     if (VAL_NO_ERROR != (retval = lookup_store(name_n, class_h, type_h,
-                            unchecked_answers, &new_answer))) {
+                            unchecked_answers, &new_answer, ns_options))) {
         VAL_CACHE_UNLOCK(&ans_rwlock);
         return retval;
     }
@@ -299,7 +294,7 @@ get_cached_rrset(struct val_query_chain *matched_q,
         VAL_CACHE_LOCK_SH(&ns_rwlock);
 
         if (VAL_NO_ERROR != (retval = lookup_store(name_n, class_h, type_h,
-                            unchecked_hints, &new_answer))) {
+                            unchecked_hints, &new_answer, 0))) {
             VAL_CACHE_UNLOCK(&ns_rwlock);
             return retval;
         }
@@ -428,76 +423,6 @@ stow_answers(struct rrset_rec **new_info, struct val_query_chain *matched_q)
     return rc;
 }
 
-/*
- * Maintain a mapping between the zone and the name server that answered 
- * data for it 
- */
-int
-store_ns_for_zone(u_char * zonecut_n, struct name_server *resp_server)
-{
-    struct zone_ns_map_t *map_e;
-
-    if (!zonecut_n || !resp_server)
-        return VAL_NO_ERROR;
-
-    VAL_CACHE_LOCK_INIT(&map_rwlock, map_rwlock_init);
-    VAL_CACHE_LOCK_EX(&map_rwlock);
-
-    for (map_e = zone_ns_map; map_e; map_e = map_e->next) {
-
-        if (!namecmp(map_e->zone_n, zonecut_n)) {
-            struct name_server *nslist = NULL;
-            /*
-             * add blindly to the list 
-             */
-            clone_ns_list(&nslist, resp_server);
-            nslist->ns_next = map_e->nslist;
-            map_e->nslist = nslist;
-            break;
-        }
-    }
-
-    if (!map_e) {
-        map_e =
-            (struct zone_ns_map_t *) MALLOC(sizeof(struct zone_ns_map_t));
-        if (map_e == NULL) {
-            VAL_CACHE_UNLOCK(&map_rwlock);
-            return VAL_OUT_OF_MEMORY;
-        }
-
-        clone_ns_list(&map_e->nslist, resp_server);
-        memcpy(map_e->zone_n, zonecut_n, wire_name_length(zonecut_n));
-        map_e->next = NULL;
-
-        if (zone_ns_map != NULL)
-            map_e->next = zone_ns_map;
-        zone_ns_map = map_e;
-    }
-
-    VAL_CACHE_UNLOCK(&map_rwlock);
-
-    return VAL_NO_ERROR;
-}
-
-static int
-free_zone_nslist(void)
-{
-    struct zone_ns_map_t *map_e;
-
-    VAL_CACHE_LOCK_INIT(&map_rwlock, map_rwlock_init);
-    VAL_CACHE_LOCK_EX(&map_rwlock);
-    while (zone_ns_map) {
-        map_e = zone_ns_map;
-        zone_ns_map = zone_ns_map->next;
-
-        if (map_e->nslist)
-            free_name_servers(&map_e->nslist);
-        FREE(map_e);
-    }
-    VAL_CACHE_UNLOCK(&map_rwlock);
-
-    return VAL_NO_ERROR;
-}
 
 /*
  * Get zone information: this could either be from 
@@ -520,7 +445,6 @@ get_nslist_from_cache(val_context_t *ctx,
     u_char       *p;
     u_int16_t     qtype;
     u_char       *qname_n;
-    struct zone_ns_map_t *map_e, *saved_map;
     u_char       *tmp_zonecut_n = NULL;
     struct timeval  tv;
 
@@ -537,41 +461,6 @@ get_nslist_from_cache(val_context_t *ctx,
     *zonecut_n = NULL;
     gettimeofday(&tv, NULL);
     
-    VAL_CACHE_LOCK_INIT(&map_rwlock, map_rwlock_init);
-    VAL_CACHE_LOCK_SH(&map_rwlock);
-
-    /*
-     * Check mapping table between zone and nameserver to see if 
-     * NS information is available here 
-     */
-    saved_map = NULL;
-    for (map_e = zone_ns_map; map_e; map_e = map_e->next) {
-
-        /*
-         * check if zone is within query 
-         */
-        if (NULL != (p = namename(qname_n, map_e->zone_n))) {
-            if (!saved_map || (namecmp(p, saved_map->zone_n) > 0)) {
-                saved_map = map_e;
-            }
-        }
-    }
-
-    if (saved_map) {
-        *zonecut_n = (u_char *) MALLOC (wire_name_length(saved_map->zone_n) *
-                sizeof (u_char));
-        if (*zonecut_n == NULL) {
-            VAL_CACHE_UNLOCK(&map_rwlock);
-            return VAL_OUT_OF_MEMORY;
-        } 
-        clone_ns_list(ref_ns_list, saved_map->nslist);
-        memcpy(*zonecut_n, saved_map->zone_n, wire_name_length(saved_map->zone_n));
-        VAL_CACHE_UNLOCK(&map_rwlock);
-        return VAL_NO_ERROR;
-    }
-
-    VAL_CACHE_UNLOCK(&map_rwlock);
-
     tmp_zonecut_n = NULL;
 
 
@@ -617,21 +506,22 @@ get_nslist_from_cache(val_context_t *ctx,
     }
 
 
-    if (name_n) {
+    if (name_n && tmp_zonecut_n) {
+
         bootstrap_referral(ctx, name_n, unchecked_hints, matched_qfq, queries,
                            ref_ns_list);
-    }
 
-    if (*ref_ns_list && tmp_zonecut_n) {
-        *zonecut_n = (u_char *) MALLOC (wire_name_length(tmp_zonecut_n) *
-                sizeof (u_char));
-        if (*zonecut_n == NULL) {
-            VAL_CACHE_UNLOCK(&ns_rwlock);
-            free_name_servers(ref_ns_list);
-            *ref_ns_list = NULL;
-            return VAL_OUT_OF_MEMORY;
-        } 
-        memcpy(*zonecut_n, tmp_zonecut_n, wire_name_length(tmp_zonecut_n));
+        if (*ref_ns_list) {
+            *zonecut_n = (u_char *) MALLOC (wire_name_length(tmp_zonecut_n) *
+                    sizeof (u_char));
+            if (*zonecut_n == NULL) {
+                VAL_CACHE_UNLOCK(&ns_rwlock);
+                free_name_servers(ref_ns_list);
+                *ref_ns_list = NULL;
+                return VAL_OUT_OF_MEMORY;
+            } 
+            memcpy(*zonecut_n, tmp_zonecut_n, wire_name_length(tmp_zonecut_n));
+        }
     }
     
     VAL_CACHE_UNLOCK(&ns_rwlock);
@@ -654,8 +544,6 @@ free_validator_cache(void)
     unchecked_answers = NULL;
     VAL_CACHE_UNLOCK(&ans_rwlock);
     
-    free_zone_nslist();
-
     return VAL_NO_ERROR;
 }
 
