@@ -15,6 +15,11 @@
 #include <ifaddrs.h>
 #endif
 
+#include <openssl/conf.h>
+#include <openssl/evp.h>
+#include <openssl/engine.h>
+#include <openssl/err.h>
+
 #include "val_support.h"
 #include "val_policy.h"
 #include "val_cache.h"
@@ -397,10 +402,13 @@ val_create_context_internal( const char *label,
             *rescur = '\0';
             rescur++;
  
+            /* Set RD unless we're asked to disable recursion */
             if (polflags & CTX_DYN_POL_RES_NRD) {
                 ns_options = SR_QUERY_NOREC;
+            } else {
+                ns_options = SR_QUERY_RECURSE;
             }
-            ns = parse_name_server(resptr, NULL, SR_QUERY_NOREC);
+            ns = parse_name_server(resptr, NULL, ns_options);
 
             /* Ignore name servers that we don't understand */
             if (ns != NULL) {
@@ -531,6 +539,8 @@ val_create_context_internal( const char *label,
 
     (*newcontext)->root_ns = NULL; 
     (*newcontext)->nslist = NULL; 
+    (*newcontext)->search = NULL; 
+    (*newcontext)->zone_ns_map = NULL; 
     (*newcontext)->dyn_polflags = polflags;
 
     (*newcontext)->dyn_valpolopt = dyn_valpolopt;
@@ -769,6 +779,9 @@ val_free_context(val_context_t * context)
     if (context->search)
         FREE(context->search);
 
+    if (context->zone_ns_map)
+        _val_free_zone_nslist(context->zone_ns_map);
+
     if (context->resolv_conf)
         FREE(context->resolv_conf);
 
@@ -837,6 +850,14 @@ val_free_validator_state()
     WSACleanup();
 #endif
 
+    /* Whole bunch of openssl cleanup routines */
+    CONF_modules_unload(1); 
+    EVP_cleanup(); 
+    ENGINE_cleanup(); 
+    CRYPTO_cleanup_all_ex_data(); 
+    ERR_remove_state(0); 
+    ERR_free_strings(); 
+
     return VAL_NO_ERROR;
 }
 
@@ -868,36 +889,226 @@ val_context_setqflags(val_context_t *context,
     }
     
     CTX_UNLOCK_ACACHE(ctx);
+
     CTX_UNLOCK_POL(ctx);
 
     return VAL_NO_ERROR;
 }  
 
 int
-val_context_ip4(val_context_t * context)
+val_is_local_trusted(val_context_t *context, int *trusted)
 {
-    if (context) {
-        /* No IPv4 if we're only configured to use IPv6 */
-        if (context->g_opt && 
-            context->g_opt->proto == VAL_POL_GOPT_PROTO_IPV6)
-            return 0;
-        return context->have_ipv4;
+    val_context_t *ctx = NULL;
+
+    if (trusted == NULL)
+        return VAL_BAD_ARGUMENT;
+
+    ctx = val_create_or_refresh_context(context); /* does CTX_LOCK_POL_SH */
+    if (ctx == NULL)
+        return VAL_INTERNAL_ERROR;
+
+    if (ctx && ctx->g_opt && ctx->g_opt->local_is_trusted)
+        *trusted = 1;
+    else
+        *trusted = 0;
+
+    CTX_UNLOCK_POL(ctx);
+
+    return VAL_NO_ERROR;    
+}
+
+
+int
+_val_store_ns_in_map(u_char * zonecut_n, struct name_server *ns, 
+                     struct zone_ns_map_t **zone_ns_map)
+{
+    struct zone_ns_map_t *map_e;
+
+    if (!zonecut_n || !ns || !zone_ns_map)
+        return VAL_BAD_ARGUMENT;
+
+    for (map_e = *zone_ns_map; map_e; map_e = map_e->next) {
+
+        if (!namecmp(map_e->zone_n, zonecut_n)) {
+            struct name_server *nslist = NULL;
+            /*
+             * add blindly to the list 
+             */
+            clone_ns_list(&nslist, ns);
+            nslist->ns_next = map_e->nslist;
+            map_e->nslist = nslist;
+            break;
+        }
     }
 
-    return 0;
+    if (!map_e) {
+        map_e =
+            (struct zone_ns_map_t *) MALLOC(sizeof(struct zone_ns_map_t));
+        if (map_e == NULL) {
+            return VAL_OUT_OF_MEMORY;
+        }
+
+        clone_ns_list(&map_e->nslist, ns);
+        memcpy(map_e->zone_n, zonecut_n, wire_name_length(zonecut_n));
+        map_e->next = NULL;
+
+        if (*zone_ns_map != NULL)
+            map_e->next = *zone_ns_map;
+        *zone_ns_map = map_e;
+    }
+
+    return VAL_NO_ERROR;
+}
+
+/*
+ * Maintain a mapping between the zone and the name server that answered 
+ * data for it 
+ */
+int
+val_context_store_ns_for_zone(val_context_t *context, char * zone, 
+                              char *resp_server, int recursive)
+{
+    struct name_server *ns;
+    int retval;
+    val_context_t *ctx;
+    struct val_query_chain *q;
+    u_char zone_n[NS_MAXCDNAME];
+    unsigned long options = SR_QUERY_RECURSE;
+
+    ctx = val_create_or_refresh_context(context); /* does CTX_LOCK_POL_SH */
+    if (ctx == NULL) { 
+        return VAL_INTERNAL_ERROR;
+    }
+
+    /* Lock exclusively */
+    CTX_LOCK_ACACHE(ctx);
+
+    if (!recursive) {
+        options |= SR_QUERY_NOREC;
+    }
+
+    if (resp_server && zone&& 
+        (-1 != ns_name_pton(zone, zone_n, sizeof(zone_n))) && 
+        (NULL != (ns = parse_name_server(resp_server, NULL, 
+                                         options)))) {
+        retval = _val_store_ns_in_map(zone_n, ns, &ctx->zone_ns_map);
+    } else {
+        retval = VAL_BAD_ARGUMENT;
+    }
+
+    /* Flush queries that match this name */
+    for(q=ctx->q_list; q; q=q->qc_next) {
+        if (NULL != namename(q->qc_name_n, zone_n)) {
+            q->qc_flags |= VAL_QUERY_MARK_FOR_DELETION;
+        }
+    }
+
+    CTX_UNLOCK_ACACHE(ctx);
+
+    CTX_UNLOCK_POL(ctx);
+
+    return retval;
 }
 
 int
-val_context_ip6(val_context_t * context)
+_val_free_zone_nslist(struct zone_ns_map_t *zone_ns_map)
 {
-    if (context) {
-        /* No IPv6 if we're only configured to use IPv4 */
-        if (context->g_opt &&
-            context->g_opt->proto == VAL_POL_GOPT_PROTO_IPV4)
-            return 0;
-        return context->have_ipv6;
+    struct zone_ns_map_t *map_e;
+
+    while (zone_ns_map) {
+        map_e = zone_ns_map;
+        zone_ns_map = zone_ns_map->next;
+
+        if (map_e->nslist)
+            free_name_servers(&map_e->nslist);
+        FREE(map_e);
     }
 
-    return 0;
+    return VAL_NO_ERROR;
 }
 
+int 
+_val_get_mapped_ns(val_context_t *ctx, 
+                   u_char *qname_n,
+                   u_int16_t qtype,
+                   u_char **zonecut_n,
+                   struct name_server **ref_ns_list) 
+{
+    struct zone_ns_map_t *map_e, *saved_map;
+    u_char *p = NULL;
+
+    if (ctx == NULL || qname_n == NULL || zonecut_n == NULL || ref_ns_list == NULL)
+        return VAL_BAD_ARGUMENT;
+
+    *zonecut_n = NULL;
+    *ref_ns_list = NULL;
+    saved_map = NULL;
+
+    for (map_e = ctx->zone_ns_map; map_e; map_e = map_e->next) {
+
+        /*
+         * check if zone is within query 
+         */
+        if (NULL != (p = namename(qname_n, map_e->zone_n))) {
+            if (p == qname_n && qtype == ns_t_ds) {
+                /* 
+                 * If we're looking for the DS, we shouldn't return an
+                 * exact match 
+                 */
+                continue;
+
+            }
+            if (!saved_map || (namecmp(p, saved_map->zone_n) > 0)) {
+                saved_map = map_e;
+            }
+        }
+    }
+
+    if (saved_map) {
+        *zonecut_n = (u_char *) MALLOC (wire_name_length(saved_map->zone_n) *
+                sizeof (u_char));
+        if (*zonecut_n == NULL) {
+            return VAL_OUT_OF_MEMORY;
+        } 
+        clone_ns_list(ref_ns_list, saved_map->nslist);
+        memcpy(*zonecut_n, saved_map->zone_n, wire_name_length(saved_map->zone_n));
+    }
+
+    return VAL_NO_ERROR;
+}
+
+int
+_val_context_ip4(val_context_t *ctx)
+{
+    int retval;
+
+    if (ctx == NULL)
+        return 0;
+
+    /* No IPv4 if we're only configured to use IPv6 */
+    if (ctx->g_opt && 
+        ctx->g_opt->proto == VAL_POL_GOPT_PROTO_IPV6) {
+        retval = 0;
+    }
+    retval = ctx->have_ipv4;
+
+    return retval;
+}
+
+int
+_val_context_ip6(val_context_t *ctx)
+{
+    int retval;
+
+    if (ctx == NULL)
+        return 0;
+
+    /* No IPv6 if we're only configured to use IPv4 */
+    if (ctx->g_opt && 
+        ctx->g_opt->proto == VAL_POL_GOPT_PROTO_IPV4) {
+        retval = 0;
+    }
+    retval = ctx->have_ipv6;
+
+    return retval;
+}
