@@ -15,7 +15,6 @@
 #include "val_context.h"
 #include "validator/val_dane.h"
 
-
 /*
  * Internal callback structure
  */
@@ -27,13 +26,83 @@ typedef struct _val_dane_async_status {
     val_async_status *das; /* helps us cancel this lookup */
 } _val_dane_async_status_t;
 
+/* 
+ * Free up the list of DANE TLSA records 
+ */
+void val_free_dane(struct val_danestatus *dres)
+{
+    struct val_danestatus *prev, *cur;
+
+    prev = NULL;
+    cur = dres;
+
+    while (cur) {
+        prev = cur;
+        cur = cur->next;
+        if (prev->data)
+            FREE(prev->data);
+        FREE(prev);
+    }
+}
+
+static int
+clone_danestatus(struct val_danestatus *dstatus,
+                 struct val_danestatus **dstatus_p)
+{
+    struct val_danestatus *dcur = NULL;
+    struct val_danestatus *dtail = NULL;
+    struct val_danestatus *dnew = NULL;
+
+    if (dstatus == NULL || dstatus_p == NULL)
+        return VAL_BAD_ARGUMENT;
+
+    *dstatus_p = NULL;
+    dcur = dstatus;
+
+    while (dcur) {
+
+        dnew = (struct val_danestatus *) MALLOC (sizeof (struct
+                    val_danestatus));
+        if (dnew == NULL)
+            goto err;
+
+        dnew->ttl = dcur->ttl;
+        dnew->usage = dcur->usage;
+        dnew->selector = dcur->selector;
+        dnew->type = dcur->type;
+        dnew->datalen = dcur->datalen;
+        dnew->data = (unsigned char *) MALLOC (dcur->datalen *
+                sizeof(unsigned char));
+        if (dnew->data == NULL) {
+            FREE(dnew);
+            goto err;
+        }
+        memcpy(dnew->data, dcur->data, dcur->datalen);
+        dnew->next = NULL;
+        if (dtail) {
+            dtail->next = dnew;
+        } else {
+            *dstatus_p = dnew;
+        }
+        dtail = dnew;
+        dcur = dcur->next;
+    }
+
+    return VAL_NO_ERROR;
+
+err:
+    val_free_dane(*dstatus_p);
+    *dstatus_p = NULL;
+    return VAL_OUT_OF_MEMORY;
+}
 
 /*
  * Construct a linked list of DANE structures from the result linked
  * list
  */
 static int 
-get_dane_from_result(struct val_daneparams *dparam,
+get_dane_from_result(val_context_t *ctx,
+                     struct val_daneparams *dparam,
                      struct val_result_chain *results,
                      struct val_danestatus **dres)
 {
@@ -57,10 +126,10 @@ get_dane_from_result(struct val_daneparams *dparam,
         if (!val_istrusted(res->val_rc_status))
             return VAL_DANE_NOTVALIDATED;
 
-        /* 
-         * not validated is okay for non-existence conditions
+        /*
+         * Ensure that all records (including aliases) are validated.
          */
-        if (!validated || !val_isvalidated(res->val_rc_status))
+        if (!val_isvalidated(res->val_rc_status))
             validated = 0;
 
         /* 
@@ -70,9 +139,18 @@ get_dane_from_result(struct val_daneparams *dparam,
             continue;
         }
 
+        /* 
+         * provably non-existence does not imply error 
+         */
         if (val_does_not_exist(res->val_rc_status))
             return VAL_DANE_IGNORE_TLSA;
 
+        /* 
+         * and neither does provably insecure conditions 
+         */
+        if (res->val_rc_status == VAL_PINSECURE)
+            return VAL_DANE_IGNORE_TLSA;
+
         /* 
          * We're now about to process the DANE record
          * DANE resource records MUST be validated 
@@ -81,8 +159,7 @@ get_dane_from_result(struct val_daneparams *dparam,
             return VAL_DANE_NOTVALIDATED;
 
         rrset = res->val_rc_rrset;
-        if(!res->val_rc_alias && rrset && 
-                rrset->val_rrset_type == ns_t_tlsa) {
+        if(rrset && rrset->val_rrset_type == ns_t_tlsa) {
             struct val_rr_rec  *rr = rrset->val_rrset_data; 
 
             while (rr) {
@@ -105,6 +182,7 @@ get_dane_from_result(struct val_daneparams *dparam,
                      * Don't have enough data for the fixed length TLSA
                      * fields
                      */
+                    FREE(dcur);
                     rc = VAL_DANE_MALFORMED_TLSA;
                     goto err;
                 }
@@ -117,18 +195,19 @@ get_dane_from_result(struct val_daneparams *dparam,
                 dcur->selector = *cp++;
                 dcur->type = *cp++;
                 dcur->datalen = end - cp;
-                dcur->next = NULL;
-                if (dcur->datalen > 0) {
-                    dcur->data = (u_char *) MALLOC (dcur->datalen * sizeof(u_char));
-                    if (dcur->data == NULL) {
-                        rc = VAL_DANE_INTERNAL_ERROR;
-                        goto err;
-                    }
-                    memcpy(dcur->data, cp, dcur->datalen);
-                } else {
-                    dcur->data = NULL;
+                if (dcur->datalen <= 0) {
+                    FREE(dcur);
+                    rc = VAL_DANE_MALFORMED_TLSA;
+                    goto err;
                 }
-
+                dcur->next = NULL;
+                dcur->data = (u_char *) MALLOC (dcur->datalen * sizeof(u_char));
+                if (dcur->data == NULL) {
+                    FREE(dcur);
+                    rc = VAL_DANE_INTERNAL_ERROR;
+                    goto err;
+                }
+                memcpy(dcur->data, cp, dcur->datalen);
                 if (dtail == NULL) {
                     /* add the head element */
                     dtail = dcur;
@@ -223,7 +302,8 @@ _dane_async_callback(val_async_status *as, int event,
     /* 
      * XXX Use struct val_daneparams to filter our results
      */
-    dane_rc = get_dane_from_result(dstat->dparam,
+    dane_rc = get_dane_from_result(ctx,
+                                   dstat->dparam,
                                    cbp->results, 
                                    &dres);
 
@@ -287,25 +367,6 @@ const char *p_dane_error(int rc)
     return err;
 }
 
-/* 
- * Free up the list of DANE TLSA records 
- */
-void val_free_dane(struct val_danestatus *dres)
-{
-    struct val_danestatus *prev, *cur;
-
-    prev = NULL;
-    cur = dres;
-
-    while (cur) {
-        prev = cur;
-        cur = cur->next;
-        if (prev->data)
-            FREE(prev->data);
-        FREE(prev);
-    }
-}
-
 /*
  * Async validation of DANE records
  */
@@ -425,7 +486,8 @@ int val_getdaneinfo(val_context_t *context,
     /* 
      * XXX Use struct val_daneparams to filter our results
      */
-    dane_rc = get_dane_from_result(params,
+    dane_rc = get_dane_from_result(ctx,
+                                   params,
                                    results, 
                                    dres);
     val_log(ctx, LOG_DEBUG,
@@ -437,27 +499,19 @@ int val_getdaneinfo(val_context_t *context,
 }
 
 static
-int get_pkeybuf(const unsigned char *data, int len, 
-                int *pkeyLen, unsigned char **pkeybuf)
+int get_pkeybuf(X509 *cert, int *pkeyLen, unsigned char **pkeybuf)
 {
-    X509 *cert;
     EVP_PKEY *pkey;
     int rv = 0;
-    const unsigned char *tmp = data;
 
-    if (pkeyLen == NULL || pkeybuf == NULL)
+    if (cert == NULL || pkeyLen == NULL || pkeybuf == NULL)
         return -1;
 
     *pkeyLen = 0;
     *pkeybuf = NULL;
 
-    cert = d2i_X509(NULL, &tmp, len);
-    if (cert == NULL)
-        return -1;
-
     pkey = X509_get_pubkey(cert);
     if (pkey == NULL) {
-        X509_free(cert);
         return -1;
     }
 
@@ -471,19 +525,135 @@ int get_pkeybuf(const unsigned char *data, int len,
         i2d_PUBKEY(pkey, &tmp2);
     }
     EVP_PKEY_free(pkey); 
-    X509_free(cert);
     return rv;
 }
 
-int val_dane_match(val_context_t *context,
-                   struct val_danestatus *dane_cur, 
+
+/*
+ * check if the qname matches any of the names provided in the given
+ * string array. Wildcards of the form *.name can also match as long as
+ * name is a substring within qname.
+ */
+static int
+do_cert_namechk(val_context_t *context,
+        char *qname, X509 *cert)
+{
+    int num;
+    int i;
+
+    GENERAL_NAMES *saNames = NULL;
+    GENERAL_NAME *cur = NULL;
+    X509_NAME *subj = NULL;
+    int retval = 1;
+    ASN1_STRING *astr;
+    char *cp;
+
+    if (qname == NULL || cert == NULL)
+        return 0;
+
+    /*
+     * XXX This should be replaced with X509_check_host() at some point
+     */
+
+    /* compare against subject alt names */
+    saNames = X509_get_ext_d2i(cert, NID_subject_alt_name, NULL, NULL);
+    if (saNames) {
+        num = sk_GENERAL_NAME_num(saNames); 
+        for (i=0; i<num; i++) {
+            cur = sk_GENERAL_NAME_value(saNames, i);
+            if (cur->type == GEN_DNS) {
+                astr = cur->d.dNSName;
+                cp = (char *)astr->data;
+
+                /* check for exact match or wildcard match */
+                if (cp &&
+                    ((!strcmp(cp, qname)) ||
+                     (*cp == '*' && *(cp+1) == '.' && 
+                                strstr(qname, cp+2)))) {
+                    retval = 1;
+                    goto done;
+                }
+                else {
+                    retval = 0;
+                    /* keep trying */
+                }
+            }
+        }
+    }
+
+    /* compare against subject name */
+    subj = X509_get_subject_name(cert);
+    i = -1;
+    while ((i = X509_NAME_get_index_by_NID(subj, NID_commonName, i)) >=0) {
+        X509_NAME_ENTRY *entry = X509_NAME_get_entry(subj, i);
+        astr = X509_NAME_ENTRY_get_data(entry);
+        cp = (char *)astr->data;
+
+        /* check for exact match or wildcard match */
+        if (cp &&
+            ((!strcmp(cp, qname)) ||
+              (*cp == '*' && *(cp+1) == '.' && 
+                  strstr(qname, cp+2)))) {
+            retval = 1;
+            goto done;
+        } else {
+            retval = 0;
+            /* keep trying */
+        }
+    }
+
+
+done:
+    if (saNames) {
+        //sk_GENERAL_NAME_pop_free(saNames, GENERAL_NAME_free);
+        GENERAL_NAMES_free(saNames);
+    }
+
+    return retval;
+}
+
+/*
+ * Do certificate name checks for DER encoded Cert
+ */
+
+int 
+val_dane_cert_namechk(val_context_t *context,
+                   char *qname,
                    const unsigned char *data, 
-                   int len)
+                   int len) 
+{
+    X509 *cert;
+    const unsigned char *tmp = data;
+    int ret;
+
+    if (data == NULL)
+        return 0;
+
+    cert = d2i_X509(NULL, &tmp, len);
+    if (cert == NULL)
+        return 0;
+
+    ret = do_cert_namechk(context, qname, cert);
+
+    X509_free(cert);
+
+    return ret;
+}
+
+/*
+ * Matches a DANE record against the correct part of a key, either in
+ * raw or a calculated hash of the part.
+ */
+static int 
+val_dane_match_internal(val_context_t *context,
+                           struct val_danestatus *dane_cur, 
+                           const unsigned char *data,
+                           int len,
+                           X509 *cert)
 {
     val_context_t *ctx;
-    int retval = VAL_DANE_CHECK_FAILED;
 
-    if (dane_cur == NULL || data == NULL)
+    if (cert == NULL || data == NULL || len <= 0 || dane_cur == NULL)
         return VAL_DANE_CHECK_FAILED;
 
     ctx = val_create_or_refresh_context(context);/* does CTX_LOCK_POL_SH */
@@ -504,8 +674,6 @@ int val_dane_match(val_context_t *context,
     }
 
     if (dane_cur->type == DANE_MATCH_EXACT) {
-        X509 *cert;
-        X509 *tlsa_cert;
 #if 0
     {
         char buf1[1028];
@@ -520,21 +688,15 @@ int val_dane_match(val_context_t *context,
 #endif
 
         if (dane_cur->selector == DANE_SEL_FULLCERT) {
-            const unsigned char *tmp = data;
-            const unsigned char *tmp2 = dane_cur->data;
-            cert = d2i_X509(NULL, &tmp, len);
-            tlsa_cert = d2i_X509(NULL, &tmp2, 
-                                 dane_cur->datalen);
-            if ((cert != NULL) && (X509_cmp(tlsa_cert, cert) == 0)) {
-                X509_free(cert);
-                X509_free(tlsa_cert);
+            if (len == dane_cur->datalen &&
+                    !memcmp(data, dane_cur->data, len)) {
+
                 val_log(ctx, LOG_INFO, "val_dane_match(): DANE_SEL_FULLCERT/DANE_MATCH_EXACT success");
                 CTX_UNLOCK_POL(ctx);
                 return VAL_DANE_NOERROR;
             }
+
             val_log(ctx, LOG_NOTICE, "val_dane_match(): DANE_SEL_FULLCERT/DANE_MATCH_EXACT failed");
-            X509_free(cert);
-            X509_free(tlsa_cert);
             CTX_UNLOCK_POL(ctx);
             return VAL_DANE_CHECK_FAILED;
 
@@ -542,7 +704,7 @@ int val_dane_match(val_context_t *context,
             int pkeyLen = 0;
             unsigned char *pkeybuf = NULL;
 
-            if (0 != get_pkeybuf(data, len, &pkeyLen, &pkeybuf)) {
+            if (0 != get_pkeybuf(cert, &pkeyLen, &pkeybuf)) {
                 CTX_UNLOCK_POL(ctx);
                 return VAL_DANE_CHECK_FAILED;
             }
@@ -572,7 +734,7 @@ int val_dane_match(val_context_t *context,
         } else {
             int pkeyLen = 0;
             unsigned char *pkeybuf = NULL;
-            if (0 != get_pkeybuf(data, len, &pkeyLen, &pkeybuf)) {
+            if (0 != get_pkeybuf(cert, &pkeyLen, &pkeybuf)) {
                 CTX_UNLOCK_POL(ctx);
                 return VAL_DANE_CHECK_FAILED;
             }
@@ -624,7 +786,7 @@ int val_dane_match(val_context_t *context,
         } else {
             int pkeyLen = 0;
             unsigned char *pkeybuf = NULL;
-            if (0 != get_pkeybuf(data, len, &pkeyLen, &pkeybuf)) {
+            if (0 != get_pkeybuf(cert, &pkeyLen, &pkeybuf)) {
                 CTX_UNLOCK_POL(ctx);
                 return VAL_DANE_CHECK_FAILED;
             }
@@ -663,153 +825,312 @@ int val_dane_match(val_context_t *context,
     return VAL_DANE_CHECK_FAILED;
 }
 
-
-int val_dane_check(val_context_t *ctx,
-                   SSL *con,
-                   struct val_danestatus *danestatus,
-                   int *do_pathval) 
+/*
+ * Match DER encoded certificate 
+ */
+int val_dane_match(val_context_t *context,
+                   struct val_danestatus *dane_cur, 
+                   const unsigned char *data, 
+                   int len) 
 {
-    STACK_OF(X509) *certList = NULL;
-    X509 *cert = NULL;
+    X509 *cert;
+    const unsigned char *tmp = data;
+    int ret;
+
+    if (data == NULL)
+        return 0;
+
+    cert = d2i_X509(NULL, &tmp, len);
+    if (cert == NULL)
+        return 0;
+
+    ret = val_dane_match_internal(context, dane_cur, 
+                                  data, len, cert);
+
+
+    X509_free(cert);
+
+    return ret;
+}
+
+static int 
+val_X509_peer_cert_verify_cb(X509_STORE_CTX *x509ctx, void *arg)
+{
+    char    buf[256];           
+    struct val_ssl_data *ssl_dane_data;
+    int err;
+    int i;
+    X509 *cert;
+    val_context_t *context;
     struct val_danestatus *dane_cur = NULL;
-    int have_dane = 0;
+    int depth = -1;
+    STACK_OF(X509) *certList = NULL;
+    int pkix_succeeded = 0;
+    int rv = VAL_DANE_CHECK_FAILED;
+    int cert_datalen = 0;
     unsigned char *cert_data = NULL;
     unsigned char *c = NULL;
-    int cert_datalen = 0;
-    int rv;
-    int i;
-    val_context_t *context;
 
-    if (con == NULL || danestatus == NULL || do_pathval == NULL)
-        return VAL_DANE_CHECK_FAILED;
+    ssl_dane_data = (struct val_ssl_data *) arg;
+    if (x509ctx == NULL || ssl_dane_data == NULL)
+        return 0;
 
-    context = val_create_or_refresh_context(ctx);/* does CTX_LOCK_POL_SH */
-    if (context == NULL)
-        return VAL_DANE_INTERNAL_ERROR;
+    cert = x509ctx->cert;
+    context = ssl_dane_data->context;
 
-    dane_cur = danestatus;
-    while(dane_cur)  {
-        have_dane = 1;
-        *do_pathval = 1;
-    
-        val_log(context, LOG_INFO, 
-                "Checking DANE {sel=%d, type=%d, usage=%d}",
-                dane_cur->selector,
-                dane_cur->type,
-                dane_cur->usage);
+    /* 
+     * look-ahead to see if we have any DANE record that requires PKIX
+     * validation. If so go ahead and do the PKIX validation
+     */
+    depth = -1;
+    certList = NULL;
+    pkix_succeeded = 0;
 
-        switch (dane_cur->usage) {
-            case DANE_USE_CA_CONSTRAINT: /*0*/ {
-                /* 
-                 * Check that the TLSA cert matches one on the cert
-                 * chain
-                 */
-                certList = SSL_get_peer_cert_chain(con);
-                if (!certList) {
-                    break;
-                }
-                for (i = 0; i < sk_X509_num(certList); i++) {
-                    cert = sk_X509_value(certList, i);
-                    cert_datalen = i2d_X509(cert, NULL);
-                    if (cert_datalen > 0) {
-                        cert_data = OPENSSL_malloc(cert_datalen);
-                        if (cert_data) {
-                            c = cert_data;
-                            cert_datalen = i2d_X509(cert, &c);
-                            if(cert_datalen > 0 &&
-                                val_dane_match(context,
-                                    dane_cur,
-                                    (const unsigned char *)cert_data,
-                                    cert_datalen) == 0) {
-                                val_log(context, LOG_INFO, "DANE: val_dane_match() success");
-                                rv = VAL_DANE_NOERROR;
-                                OPENSSL_free(cert_data);
-                                goto done;
-                            }
-                            OPENSSL_free(cert_data);
-                        }
-                    }
-                }
-                val_log(context, LOG_NOTICE, 
-                        "DANE: val_dane_check() for usage %d failed",
-                        dane_cur->usage);
-                break;
+    if(ssl_dane_data->danestatus == NULL) { 
+        return X509_verify_cert(x509ctx);
+    }
+
+    X509_NAME_oneline(X509_get_subject_name(cert), buf, sizeof(buf));
+
+    /* 
+     * Do PKIX checks only if we need to.
+     * PKIX checks requried for all types except DANE_USE_DOMAIN_ISSUED 
+     */
+    for (dane_cur = ssl_dane_data->danestatus; dane_cur; 
+            dane_cur = dane_cur->next) {
+
+        if(dane_cur->usage != DANE_USE_DOMAIN_ISSUED) {
+
+            pkix_succeeded = X509_verify_cert(x509ctx);
+            err = X509_STORE_CTX_get_error(x509ctx);
+            /*
+             * Only bypass errors related to cert issuer
+             */
+            if (err != X509_V_OK &&
+                err != X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT && 
+                err != X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY &&
+                err != X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT &&
+                err != X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN ) {
+
+                val_log(context,
+                        LOG_INFO, "DANE: cert PKIX verification failed = %s", buf);
+                return 0;
             }
 
-            case DANE_USE_DOMAIN_ISSUED: /*3*/
-                *do_pathval = 0;
-                /* fall-through */
+            certList = X509_STORE_CTX_get_chain(x509ctx);
+            depth = sk_X509_num(certList);
+            if (depth <= 0 ||
+                (!pkix_succeeded && 
+                    --depth != X509_STORE_CTX_get_error_depth(x509ctx))) {
+
+                /*
+                 * All of the above error conditions should occur at the
+                 * end of the PKIX validation chain.  If this assumption
+                 * does not hold return an error condition.
+                 *
+                 * Note that I am making the following assumptions: 
+                 *
+                 * 1) that the error returned from
+                 *    X509_STORE_CTX_get_error corresponds to the
+                 *    "closest" error (moving from the peer cert to the
+                 *    TA); i.e.  there are no other error conditions
+                 *    between depth 0 and X509_STORE_CTX_get_error_depth
+                 * 2) Other verification checks on this cert have passed;
+                 *    else the error condition would have been
+                 *    overridden by the other (more serious) error
+                 *    condition. It appears from
+                 *    openssl/crypto/x509/x509_vfy.c that this is the
+                 *    case, but I could be wrong.
+                 * 
+                 * If we realize that this understanding of the openssl
+                 * code is wrong we should disable DANE usage
+                 * type 2 till we come up with an alternative approach
+                 * for supporting TA assertion.
+                 */
+                val_log(context,
+                        LOG_WARNING, "DANE: BADSTATE X509 error depth different from cert length = %s", buf);
+                return 0;
+            }
+
+            /* we only need to do PKIX checks once */
+            break;
+        }
+    }
+
+    /* 
+     * We've either completed PKIX checks or we only have TLSA records
+     * of type DANE_USE_DOMAIN_ISSUED (in which case PKIX checks are not
+     * necessary)
+     */
+
+    /*
+     * Do certificate name checks
+     */
+    if (!do_cert_namechk(context, ssl_dane_data->qname, cert)) {
+        val_log(context,
+                LOG_WARNING, "DANE: Cert namecheck failed for %s", buf);
+        return 0;
+    }
+
+
+    dane_cur = ssl_dane_data->danestatus;
+
+    if (((cert_datalen = i2d_X509(cert, NULL)) <= 0) ||
+         ((cert_data = OPENSSL_malloc(cert_datalen)) == NULL) ||
+         (((c = cert_data)) && (cert_datalen = i2d_X509(cert, &c)) <= 0)) {
+
+        if (cert_data)
+            OPENSSL_free(cert_data);
+        return 0;
+    } 
+
+    /*
+     * Keep looking for a good TLSA match
+     */
+    while(dane_cur)  {
+        val_log(context, LOG_INFO, 
+               "Checking DANE {sel=%d, type=%d, usage=%d}",
+               dane_cur->selector,
+               dane_cur->type,
+               dane_cur->usage);
+
+        switch (dane_cur->usage) {
             case DANE_USE_SVC_CONSTRAINT: /*1*/ 
-                cert = SSL_get_peer_certificate(con);
-                cert_datalen = i2d_X509(cert, NULL);
-                if (cert_datalen > 0) {
-                    cert_data = OPENSSL_malloc(cert_datalen);
-                    if (cert_data) {
-                        c = cert_data;
-                        cert_datalen = i2d_X509(cert, &c);
-                        if(cert_datalen > 0 &&
-                                val_dane_match(context,
-                                    dane_cur,
-                                    (const unsigned char *)cert_data,
-                                    cert_datalen) == 0) {
-                            val_log(context, LOG_INFO, "DANE: val_dane_match() success");
-                            rv = VAL_DANE_NOERROR;
-                            OPENSSL_free(cert_data);
-                            goto done;
-                        }
-                        OPENSSL_free(cert_data);
-                    }
+                /* PKIX checks must pass */
+                if (!pkix_succeeded) {
+                    val_log(context,
+                            LOG_INFO, "DANE: cert PKIX verification failed = %s", buf);
+                   break; 
+                }
+                /* fall through */
+            case DANE_USE_DOMAIN_ISSUED: /*3*/
+                if (val_dane_match_internal(context,
+                        dane_cur, cert_data, cert_datalen, cert) == 0) {
+                    val_log(context, LOG_INFO, 
+                            "DANE: passed EE certificate checks = %s", buf);
+                    rv = VAL_DANE_NOERROR;
+                    goto done;
                 }
-                val_log(context, LOG_NOTICE, 
-                        "DANE: val_dane_check() for usage %d failed",
-                        dane_cur->usage);
                 break;
 
-            case DANE_USE_TA_ASSERTION: /*2*/ {
-                SSL_CTX *ctx = SSL_get_SSL_CTX(con);
-                X509_STORE *store;
-                *do_pathval = 0;
-                if (store = X509_STORE_new()) {
-                    X509 *tlsa_cert = NULL;
-                    c = dane_cur->data;
-                    tlsa_cert = d2i_X509(NULL, (const unsigned char **)&c, 
-                                         dane_cur->datalen);
-                    X509_STORE_add_cert(store, tlsa_cert);
-                    SSL_CTX_set_cert_store(ctx, store);
-                    if (SSL_get_verify_result(con) == X509_V_OK) {
-                        val_log(context, LOG_INFO, "DANE: val_dane_match() success");
+            case DANE_USE_CA_CONSTRAINT: /*0*/ 
+                /* PKIX checks must pass */
+                if (!pkix_succeeded) {
+                    val_log(context,
+                            LOG_INFO, "DANE: cert PKIX verification failed = %s", buf);
+                   break; 
+                }
+                /* fall through */
+            case DANE_USE_TA_ASSERTION: /*2*/
+                /* 
+                 * Check that the TLSA cert matches one of the certs
+                 * in the chain
+                 */
+                for (i = 0; i <= depth; i++) {
+                    cert = sk_X509_value(certList, i);
+                    if (val_dane_match_internal(context,
+                            dane_cur, cert_data, cert_datalen, cert) == 0) {
+                        /* reset err status */
+                        val_log(context, 
+                                LOG_INFO, "DANE: skipping TA PKIX validation = %s", buf);
                         rv = VAL_DANE_NOERROR;
                         goto done;
                     }
                 }
-
-                val_log(context, LOG_NOTICE, 
-                        "DANE: val_dane_check() for usage %d failed",
-                        dane_cur->usage);
                 break;
-            }
+
             default:
-                val_log(context, LOG_NOTICE, 
-                        "DANE: val_dane_check() for usage %d failed",
-                        dane_cur->usage);
                 break;
         }
 
+        val_log(context, LOG_INFO, 
+                "DANE: check for usage %d failed", dane_cur->usage);
+
         dane_cur = dane_cur->next;
     }
 
-    rv = VAL_DANE_CHECK_FAILED; 
-
 done:
-    if (have_dane) {
-        if (rv == VAL_DANE_NOERROR)
-            val_log(context, LOG_INFO, "DANE check successful");
-        else {
-            val_log(context, LOG_NOTICE, "DANE check failed");
-        }
+
+    if (cert_data)
+        OPENSSL_free(cert_data);
+
+    if (rv == VAL_DANE_NOERROR) {
+        val_log(context, LOG_NOTICE, "DANE check successful");
+        X509_STORE_CTX_set_error(x509ctx, X509_V_OK);
+        return 1;
     }
 
-    CTX_UNLOCK_POL(context);
-    return rv;
+    val_log(context, LOG_NOTICE, "DANE check failed");
+    return 0;
 }
 
+/*
+ * NOTE: This does a CTX_LOCK
+ */
+int
+val_enable_dane_ssl(val_context_t *ctx,
+                    SSL_CTX *sslctx,
+                    char *qname,
+                    struct val_danestatus *danestatus,
+                    struct val_ssl_data **ssl_dane_data)
+{
+    val_context_t *context = NULL;
+    int ret = VAL_NO_ERROR;
+    struct val_danestatus *danestatus_p = NULL;
+
+    if (sslctx == NULL || ssl_dane_data == NULL || 
+            danestatus == NULL || qname == NULL)
+        return VAL_BAD_ARGUMENT;
+
+    context = val_create_or_refresh_context(ctx);/* does CTX_LOCK_POL_SH */
+    if (context == NULL)
+        return VAL_OUT_OF_MEMORY;
+
+    *ssl_dane_data = (struct val_ssl_data *)MALLOC(sizeof(struct val_ssl_data));
+    if (*ssl_dane_data == NULL) {
+        CTX_UNLOCK_POL(context);
+        return VAL_OUT_OF_MEMORY;
+    }
+
+    (*ssl_dane_data)->qname = STRDUP(qname);
+    if ((*ssl_dane_data)->qname == NULL) {
+        FREE(*ssl_dane_data);
+        CTX_UNLOCK_POL(context);
+        return VAL_OUT_OF_MEMORY;
+    }
+
+    if (VAL_NO_ERROR != (ret = clone_danestatus(danestatus, &danestatus_p))) {
+        FREE((*ssl_dane_data)->qname);
+        FREE(*ssl_dane_data);
+        CTX_UNLOCK_POL(context);
+        return ret;
+    }
+
+    (*ssl_dane_data)->danestatus = danestatus_p;
+    (*ssl_dane_data)->context = context;
+
+    /*
+     * Callback from EE cert validation
+     */ 
+    SSL_CTX_set_cert_verify_callback(sslctx, 
+                                     val_X509_peer_cert_verify_cb,
+                                     (void *)(*ssl_dane_data));
+    /*
+     * Don't call CTX_UNLOCK_POL since we release the lock only when we
+     * free ssl_dane_data
+     */
+
+    return VAL_NO_ERROR;
+}
+
+void
+val_free_dane_ssl(struct val_ssl_data *ssl_dane_data)
+{
+    if (ssl_dane_data == NULL)
+        return;
+
+    FREE(ssl_dane_data->qname);
+    val_free_dane(ssl_dane_data->danestatus);
+    CTX_UNLOCK_POL(ssl_dane_data->context);
+    FREE(ssl_dane_data);
+}
