@@ -245,6 +245,8 @@ set_global_opt_defaults(val_global_opt_t *gopt)
     gopt->rec_fallback = 1;
     gopt->max_refresh = VAL_POL_GOPT_MAXREFRESH;
     gopt->proto = VAL_POL_GOPT_PROTO_ANY;
+    gopt->timeout = RES_TIMEOUT;
+    gopt->retry = RES_RETRY;
 }
 
 int 
@@ -279,6 +281,10 @@ update_dynamic_gopt(val_global_opt_t **g_new, val_global_opt_t *g)
         (*g_new)->max_refresh = g->max_refresh;        
     if (g->proto != VAL_POL_GOPT_UNSET)
         (*g_new)->proto = g->proto;        
+    if (g->timeout != VAL_POL_GOPT_UNSET)
+        (*g_new)->timeout = g->timeout;        
+    if (g->retry != VAL_POL_GOPT_UNSET)
+        (*g_new)->retry = g->retry;        
 
     return VAL_NO_ERROR;
 }
@@ -1103,6 +1109,62 @@ parse_proto(char **buf_ptr, char *end_ptr, int *line_number,
     return VAL_NO_ERROR;
 }
 
+static int
+parse_timeout(char **buf_ptr, char *end_ptr, int *line_number,
+            int *endst, val_global_opt_t *g_opt)
+{
+    char            token[TOKEN_MAX];
+    int retval;
+
+    if ((buf_ptr == NULL) || (*buf_ptr == NULL) || (end_ptr == NULL) || 
+        (g_opt == NULL) || (endst == NULL) || (line_number == NULL))
+        return VAL_BAD_ARGUMENT;
+
+    /* read the next token */
+    if (VAL_NO_ERROR != (retval = 
+        val_get_token(buf_ptr, end_ptr, line_number, 
+                      token, sizeof(token), endst,
+                      CONF_COMMENT, CONF_END_STMT, 0))) {
+        return retval;
+    }
+    if ((endst && (strlen(token) == 0)) ||
+        (*buf_ptr >= end_ptr)) { 
+        return VAL_CONF_PARSE_ERROR;
+    }
+
+    g_opt->timeout = strtol(token, (char **)NULL, 10);
+
+    return VAL_NO_ERROR;
+}
+
+static int
+parse_retry(char **buf_ptr, char *end_ptr, int *line_number,
+            int *endst, val_global_opt_t *g_opt)
+{
+    char            token[TOKEN_MAX];
+    int retval;
+
+    if ((buf_ptr == NULL) || (*buf_ptr == NULL) || (end_ptr == NULL) || 
+        (g_opt == NULL) || (endst == NULL) || (line_number == NULL))
+        return VAL_BAD_ARGUMENT;
+
+    /* read the next token */
+    if (VAL_NO_ERROR != (retval = 
+        val_get_token(buf_ptr, end_ptr, line_number, 
+                      token, sizeof(token), endst,
+                      CONF_COMMENT, CONF_END_STMT, 0))) {
+        return retval;
+    }
+    if ((endst && (strlen(token) == 0)) ||
+        (*buf_ptr >= end_ptr)) { 
+        return VAL_CONF_PARSE_ERROR;
+    }
+
+    g_opt->retry = strtol(token, (char **)NULL, 10);
+
+    return VAL_NO_ERROR;
+}
+
 static int
 get_global_options(char **buf_ptr, char *end_ptr, 
                    int *line_number, val_global_opt_t **g_opt) 
@@ -1197,6 +1259,20 @@ get_global_options(char **buf_ptr, char *end_ptr,
                 goto err;
             }
 
+        } else if (!strcmp(token, GOPT_TIMEOUT)) {
+            if (VAL_NO_ERROR != 
+                    (retval = parse_timeout(buf_ptr, end_ptr,
+                                          line_number, &endst, *g_opt))) {
+                goto err;
+            }
+
+        } else if (!strcmp(token, GOPT_RETRY)) {
+            if (VAL_NO_ERROR != 
+                    (retval = parse_retry(buf_ptr, end_ptr,
+                                          line_number, &endst, *g_opt))) {
+                goto err;
+            }
+
         } else {
             retval = VAL_CONF_PARSE_ERROR;
             goto err;
@@ -2048,6 +2124,16 @@ skipfileread:
         }
     }
 
+    /* if there are no global options defined set defaults here */
+    if (g_opt == NULL) {
+        g_opt = (val_global_opt_t *) MALLOC (sizeof (val_global_opt_t));
+        if (g_opt == NULL) {
+            retval = VAL_OUT_OF_MEMORY;
+            goto err;
+        }
+        set_global_opt_defaults(g_opt);
+    }
+
     /* Process Global options */
     ctx->g_opt = g_opt;
 
@@ -2152,9 +2238,9 @@ read_res_config_file(val_context_t * ctx)
     struct name_server *ns = NULL;
     unsigned long ns_options = SR_QUERY_RECURSE;
 
-    if (val_context_ip4(ctx) && !val_context_ip6(ctx)) {
+    if (_val_context_ip4(ctx) && !_val_context_ip6(ctx)) {
         ns_options |= SR_QUERY_IPV4_ONLY;
-    } else if (!val_context_ip4(ctx) && val_context_ip6(ctx)) {
+    } else if (!_val_context_ip4(ctx) && _val_context_ip6(ctx)) {
         ns_options |= SR_QUERY_IPV6_ONLY;
     }
 
@@ -2233,9 +2319,9 @@ read_res_config_file(val_context_t * ctx)
     if (ctx == NULL)
         return VAL_BAD_ARGUMENT;
 
-    if (val_context_ip4(ctx) && !val_context_ip6(ctx)) {
+    if (_val_context_ip4(ctx) && !_val_context_ip6(ctx)) {
         ns_options |= SR_QUERY_IPV4_ONLY;
-    } else if (!val_context_ip4(ctx) && val_context_ip6(ctx)) {
+    } else if (!_val_context_ip4(ctx) && _val_context_ip6(ctx)) {
         ns_options |= SR_QUERY_IPV6_ONLY;
     }
     
@@ -2405,7 +2491,7 @@ read_res_config_file(val_context_t * ctx)
                            ALL_COMMENTS, ZONE_END_STMT, 0)) &&
                 (ns_name_pton(token, zone_n, sizeof(zone_n)) != -1)) {
 
-                store_ns_for_zone(zone_n, ns);
+                retval = _val_store_ns_in_map(zone_n, ns, &ctx->zone_ns_map);
 
                 free_name_servers(&ns);
                 ns = NULL;
@@ -2421,7 +2507,7 @@ read_res_config_file(val_context_t * ctx)
             if (VAL_NO_ERROR !=
                 (retval =
                 val_get_token(&buf_ptr, end_ptr, &line_number, token, sizeof(token), &endst,
-                           ALL_COMMENTS, ZONE_END_STMT, 0))) {
+                           ALL_COMMENTS, ZONE_END_STMT, 1))) {
                 goto err;
             }
             if (ctx->search)
@@ -2524,9 +2610,9 @@ read_root_hints_file(val_context_t * ctx)
     if (ctx == NULL)
         return VAL_BAD_ARGUMENT;
    
-   if (val_context_ip4(ctx) && !val_context_ip6(ctx)) {
+   if (_val_context_ip4(ctx) && !_val_context_ip6(ctx)) {
        ipv4_only = 1;
-   } else if (!val_context_ip4(ctx) && val_context_ip6(ctx)) {
+   } else if (!_val_context_ip4(ctx) && _val_context_ip6(ctx)) {
        ipv6_only = 1;
    }
 
@@ -3129,26 +3215,3 @@ err:
     return retval;
 }
 
-int
-val_is_local_trusted(val_context_t *context, int *trusted)
-{
-    val_context_t *ctx = NULL;
-
-    if (trusted == NULL)
-        return VAL_BAD_ARGUMENT;
-
-    ctx = val_create_or_refresh_context(context); /* does CTX_LOCK_POL_SH */
-    if (ctx == NULL)
-        return VAL_INTERNAL_ERROR;
-
-    if (ctx && ctx->g_opt && ctx->g_opt->local_is_trusted)
-        *trusted = 1;
-    else
-        *trusted = 0;
-
-    CTX_UNLOCK_POL(ctx);
-
-    return VAL_NO_ERROR;    
-}
-    
-
