@@ -297,12 +297,12 @@ merge_glue_in_referral(val_context_t *context,
     pending_ns = pc->qc_referral->cur_pending_glue_ns;
     if (pending_ns) {
 
-        if (val_context_ip4(context)) {
+        if (_val_context_ip4(context)) {
             if (VAL_NO_ERROR != (retval = find_matching_glue(context, Q_WAIT_FOR_A_GLUE, qfq_pc, bucket, queries)))
                 return retval;
         }
 
-        if (val_context_ip6(context)) {
+        if (_val_context_ip6(context)) {
             if (VAL_NO_ERROR != (retval = find_matching_glue(context, Q_WAIT_FOR_AAAA_GLUE, qfq_pc, bucket, queries)))
                 return retval;
         }
@@ -392,7 +392,7 @@ merge_glue_in_referral(val_context_t *context,
                 (VAL_QUERY_GLUE_REQUEST | VAL_QUERY_DONT_VALIDATE);
 
         pc->qc_state = Q_INIT;
-        if (val_context_ip4(context)) {
+        if (_val_context_ip4(context)) {
             if (VAL_NO_ERROR != (retval = add_to_qfq_chain(context,
                                            queries, pending_ns->ns_name_n, ns_t_a,
                                            ns_c_in, flags, &added_q)))
@@ -400,7 +400,7 @@ merge_glue_in_referral(val_context_t *context,
             pc->qc_state |= Q_WAIT_FOR_A_GLUE;
         }
 #ifdef VAL_IPV6
-        if (val_context_ip6(context)) {
+        if (_val_context_ip6(context)) {
             if (VAL_NO_ERROR != (retval = add_to_qfq_chain(context,
                                            queries, pending_ns->ns_name_n, ns_t_aaaa,
                                            ns_c_in, flags, &added_q)))
@@ -595,8 +595,8 @@ res_zi_unverified_ns_list(val_context_t *context,
 
     unchecked_set = unchecked_zone_info;
     while (unchecked_set != NULL) {
-        if ((val_context_ip4(context) && unchecked_set->rrs_type_h == ns_t_a) ||
-            (val_context_ip6(context) && unchecked_set->rrs_type_h == ns_t_aaaa)) {
+        if ((_val_context_ip4(context) && unchecked_set->rrs_type_h == ns_t_a) ||
+            (_val_context_ip6(context) && unchecked_set->rrs_type_h == ns_t_aaaa)) {
             /*
              * If the owner name matches the name in an *ns_list entry...
              */
@@ -702,6 +702,8 @@ find_nslist_for_query(val_context_t * context,
     struct name_server *ns;
     u_char ns_cred = SR_CRED_UNSET;
     long edns0_size;
+    int timeout;
+    int retry;
 
     if (next_qfq == NULL)
         return VAL_BAD_ARGUMENT;
@@ -719,45 +721,68 @@ find_nslist_for_query(val_context_t * context,
         FREE(next_q->qc_zonecut_n);
     next_q->qc_zonecut_n = NULL;
 
+    /* 
+     * Don't check for mapped NS and default recursive name server if
+     * we're in iterative mode. We need this check for mapped NS since
+     * otherwise we'd be stuck in a loop if we ever tried looking for
+     * names that are delegated under the mapped zone. We need to be
+     * able to follow referrals in such cases
+     */
     if (!(next_q->qc_flags & VAL_QUERY_ITERATE) &&
-         context->nslist != NULL) {
+        !(next_q->qc_flags & VAL_QUERY_IS_ITERATING)) {
+        /*
+         * Check mapping table between zone and nameserver to see if 
+         * NS information is available here 
+         */
+        if (VAL_NO_ERROR != (ret_val = _val_get_mapped_ns(context, next_q->qc_name_n,
+                                        next_q->qc_type_h, &next_q->qc_zonecut_n, 
+                                        &ref_ns_list)))
+            return ret_val;
+
+        if (ref_ns_list != NULL) {
+            next_q->qc_ns_list = ref_ns_list;
+            val_log(context, LOG_DEBUG, 
+                    "find_nslist_for_query(): Found mapped ns for query");
+            goto done;
+        }
+
         /* 
          * if we have a default name server in our resolv.conf file, send
          * to that name server, but only if we are not forcing recursion
          */
-        clone_ns_list(&(next_q->qc_ns_list), context->nslist);
+        if (context->nslist != NULL) {
+            clone_ns_list(&(next_q->qc_ns_list), context->nslist);
 
-        goto done;
-    } 
-
-    ret_val = get_nslist_from_cache(context, next_qfq, queries, &ref_ns_list, &next_q->qc_zonecut_n, &ns_cred);
-    
-    if (ret_val == VAL_NO_ERROR) {
-        /* if any one is NULL, get rid of both */
-        if (next_q->qc_zonecut_n == NULL) {
-            free_name_servers(&ref_ns_list);
-            ref_ns_list = NULL;
-        } else if (ref_ns_list == NULL) {
-            if (next_q->qc_zonecut_n)
-                FREE(next_q->qc_zonecut_n);
-            next_q->qc_zonecut_n = NULL;
-        } else {
-            next_q->qc_ns_list = ref_ns_list;
-            val_log(context, LOG_DEBUG, 
-                "find_nslist_for_query(): Found cached ns_list with cred = %d.", ns_cred);
-            /* 
-             * If our answer was from an authoritative server, we
-             * also set the flag to denote that we are doing an
-             * iterative lookup
-             */
-            if (ns_cred < SR_CRED_NONAUTH)
-                next_q->qc_flags |= VAL_QUERY_ITERATE;
-            goto done; 
-        } 
+            goto done;
+        }
     } 
 
     /*
-     * work downward from root 
+     * Next look in our cache
+     */
+    if (VAL_NO_ERROR != (ret_val = get_nslist_from_cache(context,
+                    next_qfq, queries, &ref_ns_list,
+                    &next_q->qc_zonecut_n, &ns_cred))) {
+
+        return ret_val;
+    }
+    if (ref_ns_list != NULL) {
+        next_q->qc_ns_list = ref_ns_list;
+        val_log(context, LOG_DEBUG, 
+                "find_nslist_for_query(): Found cached ns_list with cred = %d.", ns_cred);
+        /* 
+         * If our answer was from an authoritative server, we
+         * also set the flag to denote that we are doing an
+         * iterative lookup
+         */
+        if (ns_cred < SR_CRED_NONAUTH)
+            next_q->qc_flags |= VAL_QUERY_IS_ITERATING;
+
+        goto done; 
+    } 
+
+    /*
+     * if all fails, work downward from root 
      */
     if (context->root_ns == NULL) {
         /*
@@ -767,7 +792,7 @@ find_nslist_for_query(val_context_t * context,
                 "find_nslist_for_query(): Trying to answer query recursively, but no root hints file found.");
         return VAL_CONF_NOT_FOUND;
     }
-    next_q->qc_flags |= VAL_QUERY_ITERATE;
+    next_q->qc_flags |= VAL_QUERY_IS_ITERATING;
     clone_ns_list(&next_q->qc_ns_list, context->root_ns);
     next_q->qc_zonecut_n = (u_char *) MALLOC(sizeof(u_char));
     if (next_q->qc_zonecut_n == NULL) {
@@ -787,9 +812,15 @@ done:
                     context->g_opt->edns0_size : RES_EDNS0_DEFAULT;
     val_log(context, LOG_DEBUG,
             "find_nslist_for_query(): Enabling DNSSEC for query (EDNS0 = %ld).", edns0_size);
+    timeout = (context && context->g_opt)?
+                    context->g_opt->timeout : RES_TIMEOUT;
+    retry = (context && context->g_opt)?
+                    context->g_opt->retry : RES_RETRY;
     for (ns = next_q->qc_ns_list; ns; ns = ns->ns_next) {
         ns->ns_edns0_size = edns0_size;
         ns->ns_options |= SR_QUERY_VALIDATING_STUB_FLAGS;
+        ns->ns_retrans = timeout;
+        ns->ns_retry = retry;
     }
 
     return VAL_NO_ERROR;
@@ -836,7 +867,7 @@ bootstrap_referral(val_context_t *context,
         }
         clone_ns_list(ref_ns_list, context->root_ns);
         matched_q->qc_state = Q_INIT;
-        matched_q->qc_flags |= VAL_QUERY_ITERATE;
+        matched_q->qc_flags |= VAL_QUERY_IS_ITERATING;
         return VAL_NO_ERROR;
     }
     
@@ -875,6 +906,11 @@ bootstrap_referral(val_context_t *context,
         matched_q->qc_referral->cur_pending_glue_ns = NULL;
         matched_q->qc_referral->pending_glue_ns = NULL;
 
+        if (matched_q->qc_referral->saved_zonecut_n) {
+            FREE(matched_q->qc_referral->saved_zonecut_n);
+            matched_q->qc_referral->saved_zonecut_n = NULL;
+        }
+
         /* save a copy of the current zonecut in the delegation info */
         if(referral_zone_n) {
             size_t len = wire_name_length(referral_zone_n); 
@@ -891,7 +927,6 @@ bootstrap_referral(val_context_t *context,
                     len);
         }
 
-
         if (*ref_ns_list != NULL) {
 
             /* save the pending list for some future occasion */
@@ -908,7 +943,7 @@ bootstrap_referral(val_context_t *context,
             free_name_servers(&pending_glue);
             if (context->root_ns != NULL) {
                 clone_ns_list(ref_ns_list, context->root_ns);
-                matched_q->qc_flags |= VAL_QUERY_ITERATE;
+                matched_q->qc_flags |= VAL_QUERY_IS_ITERATING;
                 matched_q->qc_state = Q_INIT;
             }
 
@@ -925,7 +960,7 @@ bootstrap_referral(val_context_t *context,
                         (VAL_QUERY_GLUE_REQUEST | VAL_QUERY_DONT_VALIDATE);
             matched_q->qc_state = Q_INIT;
 
-            if (val_context_ip4(context)) {
+            if (_val_context_ip4(context)) {
                 matched_q->qc_state |= Q_WAIT_FOR_A_GLUE;
                 if (VAL_NO_ERROR != (ret_val = add_to_qfq_chain(context,
                                        queries, pending_glue->ns_name_n, ns_t_a,
@@ -933,7 +968,7 @@ bootstrap_referral(val_context_t *context,
                     return ret_val;
             }
 #ifdef VAL_IPV6
-            if (val_context_ip6(context)) {
+            if (_val_context_ip6(context)) {
                 matched_q->qc_state |= Q_WAIT_FOR_AAAA_GLUE;
                 if (VAL_NO_ERROR != (ret_val = add_to_qfq_chain(context,
                                        queries, pending_glue->ns_name_n, ns_t_aaaa,
@@ -1084,7 +1119,7 @@ follow_referral_or_alias_link(val_context_t * context,
         return VAL_NO_ERROR;
     } 
 
-    matched_q->qc_flags |= VAL_QUERY_ITERATE;
+    matched_q->qc_flags |= VAL_QUERY_IS_ITERATING;
 
     res_sq_free_rrset_recs(proofs);
     *proofs = NULL;
@@ -1763,10 +1798,10 @@ digest_response(val_context_t * context,
             isrelv = 0;
         }
 
-        authoritive = (matched_q->qc_flags & VAL_QUERY_ITERATE) &&
+        authoritive = (matched_q->qc_flags & VAL_QUERY_IS_ITERATING) &&
                       (header->aa == 1);
 
-        iterative = matched_q->qc_flags & VAL_QUERY_ITERATE;
+        iterative = matched_q->qc_flags & VAL_QUERY_IS_ITERATING;
         /*
          * If it is from the answer section, it may be an alias 
          * If it is from the authority section, it may be a proof or a referral 
@@ -1891,8 +1926,8 @@ digest_response(val_context_t * context,
                                 type_h, set_type_h, class_h, ttl_h, hptr,
                                 rdata, rdata_len_h, from_section,
                                 authoritive, iterative, rrs_zonecut_n);
-            } else if ((val_context_ip4(context) && set_type_h == ns_t_a) || 
-                       (val_context_ip6(context) && set_type_h == ns_t_aaaa)) {
+            } else if ((_val_context_ip4(context) && set_type_h == ns_t_a) || 
+                       (_val_context_ip6(context) && set_type_h == ns_t_aaaa)) {
                 SAVE_RR_TO_LIST(resp_ns,
                                 &learned_zones, name_n,
                                 type_h, set_type_h, class_h, ttl_h, hptr,
@@ -2135,6 +2170,10 @@ digest_response(val_context_t * context,
             consume_referral_data(&matched_q->qc_referral, di_response, qnames);
         } 
 
+        /* reset the iterating flag if set */
+        if (matched_q->qc_flags & VAL_QUERY_IS_ITERATING)
+            matched_q->qc_flags ^= VAL_QUERY_IS_ITERATING;
+
         matched_q->qc_state = Q_ANSWERED;
         ret_val = VAL_NO_ERROR;
 
@@ -2299,7 +2338,7 @@ val_resquery_rcv(val_context_t * context,
 
     /** if there was no answer, try smaller edns0 size */
     if (ret_val == SR_NO_ANSWER) {
-        val_res_nsfallback(context, matched_q, server, name_p, closest_event);
+        val_res_nsfallback(context, matched_q, server, closest_event);
         if (response_data)
             FREE(response_data);
         if (server)
@@ -2332,7 +2371,7 @@ val_res_cancel(struct val_query_chain *matched_q)
 
 void
 val_res_nsfallback(val_context_t *context, struct val_query_chain *matched_q,
-                   struct name_server *server, const char *name_p,
+                   struct name_server *server, 
                    struct timeval *closest_event)
 {
     int ret_val;
@@ -2351,14 +2390,10 @@ val_res_nsfallback(val_context_t *context, struct val_query_chain *matched_q,
 
 #ifndef VAL_NO_ASYNC
     if (matched_q->qc_ea)
-        ret_val = res_nsfallback_ea(matched_q->qc_ea, closest_event, server,
-                                    name_p, matched_q->qc_class_h, 
-                                    matched_q->qc_type_h);
+        ret_val = res_nsfallback_ea(matched_q->qc_ea, closest_event, server);
     else
 #endif
-        ret_val = res_nsfallback(matched_q->qc_trans_id, closest_event, server,
-                                 name_p, matched_q->qc_class_h, 
-                                 matched_q->qc_type_h);
+        ret_val = res_nsfallback(matched_q->qc_trans_id, closest_event, server);
     if (ret_val < 0) {
         matched_q->qc_state = Q_RESPONSE_ERROR;
         val_res_cancel(matched_q);
@@ -2431,7 +2466,7 @@ _process_rcvd_response(val_context_t * context,
         free_domain_info_ptrs(*response);
         FREE(*response);
         *response = NULL;
-        val_res_nsfallback(context, matched_q, server, name_p, closest_event);
+        val_res_nsfallback(context, matched_q, server, closest_event);
         if (matched_q->qc_state != Q_RESPONSE_ERROR)
             matched_q->qc_state = Q_SENT;
     }
@@ -2472,6 +2507,7 @@ val_resquery_async_send(val_context_t * context,
     char            name_p[NS_MAXDNAME];
     char            name_buf[INET6_ADDRSTRLEN + 1];
     struct val_query_chain *matched_q;
+    struct timeval now;
 
     if ((matched_qfq == NULL) || (matched_qfq->qfq_query->qc_ns_list == NULL))
         return VAL_BAD_ARGUMENT;
@@ -2499,6 +2535,12 @@ val_resquery_async_send(val_context_t * context,
         }
     }
 
+    /*
+     * Update the qc_last_sent timestamp
+     */
+    gettimeofday(&now, NULL);
+    matched_q->qc_last_sent = now.tv_sec;
+
     matched_q->qc_ea = res_async_query_send(name_p, matched_q->qc_type_h,
                                             matched_q->qc_class_h, 
                                             matched_q->qc_ns_list);
@@ -2557,7 +2599,7 @@ val_resquery_async_rcv(val_context_t * context,
 
     /** if there was no answer, try smaller edns0 size */
     if (ret_val == SR_NO_ANSWER) {
-        val_res_nsfallback(context, matched_q, server, name_p, closest_event);
+        val_res_nsfallback(context, matched_q, server, closest_event);
         if (response_data)
             FREE(response_data);
         if (server)
