@@ -16,7 +16,7 @@
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
 /*
- * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2013 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #include "validator-internal.h"
@@ -35,7 +35,7 @@ namename(u_char * big_name, u_char * little_name)
     if (*little_name == '\0') {
         size_t d = wire_name_length(p);
         if (d >= 1)
-            return p+wire_name_length(p)-1;
+            return p+d-1;
         return NULL;
     }
     
@@ -68,11 +68,12 @@ base32hex_encode(u_char * in, size_t inlen, u_char ** out,
     u_char       *out_ch;
     u_char        padbuf[5];
     size_t        i, rem, extra;
+    int           len = inlen;
 
     *out = NULL;
     *outlen = 0;
 
-    if ((in == NULL) || (inlen <= 0))
+    if ((in == NULL) || (inlen == 0))
         return;
 
     /*
@@ -94,17 +95,18 @@ base32hex_encode(u_char * in, size_t inlen, u_char ** out,
     memset(padbuf, 0, 5);
     in_ch = in;
 
-    while (inlen > 0) {
+    len = inlen;
+    while (len > 0) {
 
-        if (inlen - 5 < 0) {
+        if (len - 5 < 0) {
             /*
              * pad with zeros 
              */
             i = 0;
-            while (inlen > 0) {
+            while (len > 0) {
                 padbuf[i++] = *in_ch;
                 in_ch++;
-                inlen--;
+                len--;
             }
             buf = padbuf;
         } else {
@@ -113,7 +115,7 @@ base32hex_encode(u_char * in, size_t inlen, u_char ** out,
              */
             buf = in_ch;
             in_ch += 5;
-            inlen -= 5;
+            len -= 5;
         }
 
         /*
@@ -174,7 +176,7 @@ wire_name_labels(const u_char * field)
 }
 
 void
-res_sq_free_rr_recs(struct val_rr_rec **rr)
+res_sq_free_rr_recs(struct rrset_rr **rr)
 {
     if (rr == NULL)
         return;
@@ -321,7 +323,7 @@ int
 add_to_set(struct rrset_rec *rr_set, size_t rdata_len_h,
            u_char * rdata)
 {
-    struct val_rr_rec  *rr;
+    struct rrset_rr  *rr;
 
     if ((rr_set == NULL) || (rdata == NULL) || (rdata_len_h == 0))
         return VAL_BAD_ARGUMENT;
@@ -329,7 +331,7 @@ add_to_set(struct rrset_rec *rr_set, size_t rdata_len_h,
     /*
      * Make sure we got the memory for it 
      */
-    rr = (struct val_rr_rec *) MALLOC(sizeof(struct val_rr_rec));
+    rr = (struct rrset_rr *) MALLOC(sizeof(struct rrset_rr));
     if (rr == NULL)
         return VAL_OUT_OF_MEMORY;
 
@@ -345,7 +347,7 @@ add_to_set(struct rrset_rec *rr_set, size_t rdata_len_h,
     if (rr_set->rrs_data == NULL) {
         rr_set->rrs_data = rr;
     } else {
-        struct val_rr_rec  *tmp_rr;
+        struct rrset_rr  *tmp_rr;
         tmp_rr = rr_set->rrs_data;
         while (tmp_rr->rr_next)
             tmp_rr = tmp_rr->rr_next;
@@ -368,7 +370,7 @@ int
 add_as_sig(struct rrset_rec *rr_set, size_t rdata_len_h,
            u_char * rdata)
 {
-    struct val_rr_rec  *rr;
+    struct rrset_rr  *rr;
 
     if ((rr_set == NULL) || (rdata == NULL) || (rdata_len_h == 0))
         return VAL_BAD_ARGUMENT;
@@ -376,7 +378,7 @@ add_as_sig(struct rrset_rec *rr_set, size_t rdata_len_h,
     /*
      * Make sure we got the memory for it 
      */
-    rr = (struct val_rr_rec *) MALLOC(sizeof(struct val_rr_rec));
+    rr = (struct rrset_rr *) MALLOC(sizeof(struct rrset_rr));
     if (rr == NULL)
         return VAL_OUT_OF_MEMORY;
 
@@ -393,7 +395,7 @@ add_as_sig(struct rrset_rec *rr_set, size_t rdata_len_h,
          * If this code is executed, then there is a problem brewing.
          * It will be caught in pre_verify to keep the code level.
          */
-        struct val_rr_rec  *tmp_rr;
+        struct rrset_rr  *tmp_rr;
         tmp_rr = rr_set->rrs_sig;
         while (tmp_rr->rr_next)
             tmp_rr = tmp_rr->rr_next;
@@ -509,7 +511,7 @@ init_rr_set(struct rrset_rec *new_set, u_char * name_n,
     return VAL_NO_ERROR;
 }
 
-#define IS_THE_ONE(a,n,l,t,s,c,r) \
+#define IS_THE_ONE(a,n,t,s,c,r) \
 (                                                                             \
     a &&                                     /* If there's a record */        \
     (                                                                         \
@@ -559,7 +561,6 @@ find_rr_set(struct name_server *respondent_server,
     struct rrset_rec *tryit;
     struct rrset_rec *last;
     struct rrset_rec *new_one;
-    size_t             name_len;
 
     if ((the_list == NULL) || (name_n == NULL))
         return NULL;
@@ -569,7 +570,6 @@ find_rr_set(struct name_server *respondent_server,
      */
     tryit = *the_list;
     last = NULL;
-    name_len = wire_name_length(name_n);
 
     while (tryit) {
         /*
@@ -582,7 +582,7 @@ find_rr_set(struct name_server *respondent_server,
          * <hash>.example.com in the parent zone and
          * <hash>.a.example.com in the child zone
          */
-        if (IS_THE_ONE(tryit, name_n, name_len, type_h, set_type_h,
+        if (IS_THE_ONE(tryit, name_n, type_h, set_type_h,
                        class_h, rdata_n))
             break;
 
@@ -689,6 +689,7 @@ decompress(u_char ** rdata,
     case ns_t_a6: 
     case ns_t_naptr:
     case ns_t_nsec:
+    case ns_t_tlsa:
     default:
         new_size = (size_t) * rdata_len_h;
         if (new_size == 0)
@@ -1022,7 +1023,7 @@ lower_name(u_char rdata[], size_t * index)
 
     length = wire_name_length(&rdata[(*index)]);
 
-    while ((*index) < length) {
+    while (length-- > 0) {
         rdata[(*index)] = tolower(rdata[(*index)]);
         (*index)++;
     }
@@ -1064,6 +1065,7 @@ lower(u_int16_t type_h, u_char * rdata, size_t len)
     case ns_t_naptr:
     case ns_t_nsec: 
     case ns_t_rrsig: 
+    case ns_t_tlsa:
     default:
 
         return;
@@ -1140,18 +1142,18 @@ lower(u_int16_t type_h, u_char * rdata, size_t len)
 
 
 
-struct val_rr_rec  *
-copy_rr_rec(u_int16_t type_h, struct val_rr_rec *r, int dolower)
+struct rrset_rr  *
+copy_rr_rec(u_int16_t type_h, struct rrset_rr *r, int dolower)
 {
     /*
      * Make a copy of an RR, lowering the case of any contained
      * domain name in the RR section.
      */
-    struct val_rr_rec  *the_copy;
+    struct rrset_rr  *the_copy;
 
     if (r == NULL)
         return NULL;
-    the_copy = (struct val_rr_rec *) MALLOC(sizeof(struct val_rr_rec));
+    the_copy = (struct rrset_rr *) MALLOC(sizeof(struct rrset_rr));
 
     if (the_copy == NULL)
         return NULL;
@@ -1176,44 +1178,8 @@ copy_rr_rec(u_int16_t type_h, struct val_rr_rec *r, int dolower)
     return the_copy;
 }
 
-/*
- * copy the entire list of rr_recs
- *
- * see copy_rr_rec() to copy a single val_rr_rec
- */
-struct val_rr_rec  *
-copy_rr_rec_list(u_int16_t type_h, struct val_rr_rec *o_rr, int dolower)
-{
-    struct val_rr_rec  *n_rr, *n_head;
-
-    if (NULL == o_rr)
-        return NULL;
-
-    /*
-     * copy list head
-     */
-    n_head = n_rr = copy_rr_rec(type_h, o_rr, dolower);
-    if (NULL == n_rr)
-        return NULL;
-
-    /*
-     * loop over list and copy each record
-     */
-    while (o_rr->rr_next) {
-        n_rr->rr_next = copy_rr_rec(type_h, o_rr->rr_next, dolower);
-        if (NULL == n_rr->rr_next)
-            break;
-
-        o_rr = o_rr->rr_next;
-        n_rr = n_rr->rr_next;
-    }
-    n_rr->rr_next = NULL;
-
-    return n_head;
-}
-
 int
-link_rr(struct val_rr_rec **cs, struct val_rr_rec *cr)
+link_rr(struct rrset_rr **cs, struct rrset_rr *cr)
 {
     /*
      * Insert a copied RR into the set being prepared for signing.  This
@@ -1221,7 +1187,7 @@ link_rr(struct val_rr_rec **cs, struct val_rr_rec *cr)
      */
     int             ret_val;
     int             length;
-    struct val_rr_rec  *temp_rr;
+    struct rrset_rr  *temp_rr;
 
     if (cs == NULL)
         return 0;
@@ -1306,9 +1272,9 @@ struct rrset_rec *
 copy_rrset_rec(struct rrset_rec *rr_set)
 {
     struct rrset_rec *copy_set;
-    struct val_rr_rec  *orig_rr;
-    struct val_rr_rec  *copy_rr;
-    size_t          o_length;
+    struct rrset_rr  *orig_rr;
+    struct rrset_rr  *copy_rr;
+    size_t o_length;
 
     if (rr_set == NULL)
         return NULL;
@@ -1562,7 +1528,7 @@ merge_rrset_recs(struct rrset_rec **dest, struct rrset_rec *new_info)
     struct rrset_rec *new_rr, *prev;
     struct rrset_rec *old;
     struct rrset_rec *trail_new;
-    struct val_rr_rec  *rr_exchange;
+    struct rrset_rr  *rr_exchange;
 
     if (new_info == NULL)
         return;
@@ -1640,3 +1606,54 @@ merge_rrset_recs(struct rrset_rec **dest, struct rrset_rec *new_info)
 
     return;
 }
+
+int
+val_create_rr_otw( char *name, 
+                   int type,
+                   int class,
+                   long ttl,
+                   size_t rdatalen, 
+                   u_char *rdata,
+                   size_t *buflen, 
+                   u_char **buf)
+{
+    u_char domain_name_n[NS_MAXCDNAME];
+    int ret;
+    u_char *cp = NULL;
+    size_t namelen  = 0;
+    u_int16_t class_h = (u_int16_t) class;
+    u_int16_t type_h = (u_int16_t) type;
+    u_int16_t ttl_h = (u_int32_t) ttl;
+
+    if (name == NULL || rdata == NULL || buflen == NULL || buf == NULL)
+        return VAL_BAD_ARGUMENT;
+
+    *buflen = 0;
+
+    if ((ret = ns_name_pton(name, domain_name_n, NS_MAXCDNAME)) == -1) {
+        return VAL_BAD_ARGUMENT;
+    }
+    
+    namelen = wire_name_length(domain_name_n);
+    *buflen = namelen + sizeof(u_int16_t) + sizeof(u_int16_t) +
+        sizeof(u_int32_t) + sizeof(u_int16_t) + rdatalen;
+
+    *buf = (u_char *) MALLOC (*buflen * sizeof(u_char));
+    if (*buf == NULL)
+        return VAL_OUT_OF_MEMORY;
+
+    cp = *buf;
+
+    memcpy(cp, domain_name_n, namelen);
+    cp += namelen;
+
+    NS_PUT16(type_h, cp);
+    NS_PUT16(class_h, cp);
+    NS_PUT32(ttl_h, cp);
+    NS_PUT16(rdatalen, cp);
+    memcpy(cp, rdata, rdatalen);
+
+    return VAL_NO_ERROR;
+
+}
+
