@@ -16,7 +16,7 @@
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
 /*
- * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2013 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #ifndef VAL_SUPPORT_H
@@ -38,16 +38,16 @@
 
 #define ALLOCATE_REFERRAL_BLOCK(ref) do{ \
 		ref = (struct delegation_info *) MALLOC (sizeof(struct delegation_info)); \
-		if (ref == NULL) \
-			return VAL_OUT_OF_MEMORY; \
-		ref->queries = NULL; \
-		ref->answers = NULL; \
-		ref->proofs = NULL; \
-		ref->qnames = NULL; \
-		ref->pending_glue_ns = NULL; \
-		ref->cur_pending_glue_ns = NULL; \
-		ref->saved_zonecut_n = NULL; \
-		ref->learned_zones = NULL; \
+		if (ref) { \
+		    ref->queries = NULL; \
+		    ref->answers = NULL; \
+		    ref->proofs = NULL; \
+		    ref->qnames = NULL; \
+		    ref->pending_glue_ns = NULL; \
+		    ref->cur_pending_glue_ns = NULL; \
+		    ref->saved_zonecut_n = NULL; \
+		    ref->learned_zones = NULL; \
+        }\
 } while(0)
 
 
@@ -67,7 +67,7 @@ void            base32hex_encode(u_char * in, size_t inlen,
 size_t          wire_name_labels(const u_char * field);
 size_t          wire_name_length(const u_char * field);
 
-void            res_sq_free_rr_recs(struct val_rr_rec **rr);
+void            res_sq_free_rr_recs(struct rrset_rr **rr);
 void            res_sq_free_rrset_recs(struct rrset_rec **set);
 int             add_to_qname_chain(struct qname_chain **qnames,
                                    const u_char * name_n);
@@ -122,11 +122,9 @@ int             extract_from_rr(u_char * response,
                                 size_t *rdata_index);
 void            lower_name(u_char rdata[], size_t * index);
 void            lower(u_int16_t type_h, u_char * rdata, size_t len);
-struct val_rr_rec  *copy_rr_rec(u_int16_t type_h, struct val_rr_rec *r,
+struct rrset_rr  *copy_rr_rec(u_int16_t type_h, struct rrset_rr *r,
                             int dolower);
-struct val_rr_rec  *copy_rr_rec_list(u_int16_t type_h, struct val_rr_rec *o_rr,
-                                 int dolower);
-int             link_rr(struct val_rr_rec **cs, struct val_rr_rec *cr);
+int             link_rr(struct rrset_rr **cs, struct rrset_rr *cr);
 struct rrset_rec *copy_rrset_rec(struct rrset_rec *rr_set);
 struct rrset_rec *copy_rrset_rec_list(struct rrset_rec *rr_set);
 #if 0
