@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2013 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 /*
@@ -207,12 +207,22 @@ val_sigverify(val_context_t * ctx,
     case ALG_NSEC3_RSASHA1:
 #endif
     case ALG_RSASHA1:
+#ifdef HAVE_SHA_2
     case ALG_RSASHA256:
     case ALG_RSASHA512:
+#endif
         rsasha_sigverify(ctx, data, data_len, dnskey, rrsig,
                           dnskey_status, sig_status);
         break;
 
+#if defined(HAVE_ECDSA) && defined(HAVE_OPENSSL_ECDSA_H)
+    case ALG_ECDSAP256SHA256:
+    case ALG_ECDSAP384SHA384:
+        ecdsa_sigverify(ctx, data, data_len, dnskey, rrsig,
+                        dnskey_status, sig_status);
+        break;
+#endif
+
     default:
         val_log(ctx, LOG_INFO, "val_sigverify(): Unsupported algorithm %d.",
                 rrsig->algorithm);
@@ -251,7 +261,7 @@ static int
 predict_sigbuflength(struct rrset_rec *rr_set,
                      size_t * field_length, size_t *signer_length)
 {
-    struct val_rr_rec  *rr;
+    struct rrset_rr  *rr;
     int             owner_length;
 
     /** Input has already been NULL-checked **/
@@ -278,9 +288,9 @@ static int
 make_sigfield(u_char ** field,
               size_t * field_length,
               struct rrset_rec *rr_set,
-              struct val_rr_rec *rr_sig, int is_a_wildcard)
+              struct rrset_rr *rr_sig, int is_a_wildcard)
 {
-    struct val_rr_rec  *curr_rr;
+    struct rrset_rr  *curr_rr;
     size_t          index;
     size_t          signer_length;
     size_t          owner_length;
@@ -426,7 +436,7 @@ make_sigfield(u_char ** field,
  * the rrsig
  */
 static int
-identify_key_from_sig(struct val_rr_rec *sig, u_char ** name_n,
+identify_key_from_sig(struct rrset_rr *sig, u_char ** name_n,
                       u_int16_t * footprint_n)
 {
     if ((sig == NULL) || (sig->rr_rdata == NULL) || (name_n == NULL) ||
@@ -453,7 +463,7 @@ do_verify(val_context_t * ctx,
           val_astatus_t * dnskey_status,
           val_astatus_t * sig_status,
           struct rrset_rec *the_set,
-          struct val_rr_rec *the_sig,
+          struct rrset_rr *the_sig,
           val_dnskey_rdata_t * the_key, int is_a_wildcard,
           u_int32_t flags)
 {
@@ -461,13 +471,12 @@ do_verify(val_context_t * ctx,
      * Use the crypto routines to verify the signature
      */
 
-    u_char       *ver_field;
+    u_char       *ver_field = NULL;
     size_t          ver_length;
     int             ret_val;
     val_rrsig_rdata_t rrsig_rdata;
     int clock_skew = 0;
     u_int32_t ttl_x = 0;
-    int retval = 0;
 
     /*
      * Wildcard expansions for DNSKEYs and DSs are not permitted
@@ -488,6 +497,8 @@ do_verify(val_context_t * ctx,
         val_log(ctx, LOG_INFO, 
                 "do_verify(): Could not construct signature field for verification: %s", 
                 p_val_err(ret_val));
+        if (ver_field)
+            FREE(ver_field);
         *sig_status = VAL_AC_INVALID_RRSIG;
         return 0;
     }
@@ -521,7 +532,7 @@ do_verify(val_context_t * ctx,
     /*
      * Perform the verification 
      */
-    retval = val_sigverify(ctx, is_a_wildcard, ver_field, ver_length, the_key,
+    ret_val = val_sigverify(ctx, is_a_wildcard, ver_field, ver_length, the_key,
                   &rrsig_rdata, dnskey_status, sig_status, clock_skew);
 
     if (rrsig_rdata.signature != NULL) {
@@ -530,7 +541,7 @@ do_verify(val_context_t * ctx,
     }
 
     FREE(ver_field);
-    return retval;
+    return ret_val;
 }
 
 /*
@@ -540,7 +551,7 @@ int
 ds_hash_is_equal(val_context_t *ctx,
                  u_char ds_hashtype, u_char * ds_hash,
                  size_t ds_hash_len, u_char * name_n,
-                 struct val_rr_rec *dnskey, val_astatus_t * ds_status)
+                 struct rrset_rr *dnskey, val_astatus_t * ds_status)
 {
     if ((dnskey == NULL) || (ds_hash == NULL) || (name_n == NULL)) {
         val_log(ctx, LOG_INFO, "ds_hash_is_equal(): Cannot compare DS data - invalid content");
@@ -557,12 +568,17 @@ ds_hash_is_equal(val_context_t *ctx,
 
     } 
 
-#ifdef HAVE_SHA_256
+#ifdef HAVE_SHA_2
     else if (ds_hashtype == ALG_DS_HASH_SHA256) {
         return ds_sha256_hash_is_equal(name_n, dnskey->rr_rdata,
                                        (size_t)dnskey->rr_rdata_length, 
                                        ds_hash, ds_hash_len);
     } 
+    else if (ds_hashtype == ALG_DS_HASH_SHA384) {
+        return ds_sha384_hash_is_equal(name_n, dnskey->rr_rdata,
+                                       (size_t)dnskey->rr_rdata_length, 
+                                       ds_hash, ds_hash_len);
+    } 
 #endif
 
     /* else */
@@ -574,7 +590,7 @@ ds_hash_is_equal(val_context_t *ctx,
 
 static int
 check_label_count(struct rrset_rec *the_set,
-                  struct val_rr_rec *the_sig, int *is_a_wildcard)
+                  struct rrset_rr *the_sig, int *is_a_wildcard)
 {
     size_t        owner_labels;
     size_t        sig_labels;
@@ -613,8 +629,7 @@ check_label_count(struct rrset_rec *the_set,
             savedstatus == VAL_AC_TRUST ||\
             savedstatus == VAL_AC_VERIFIED ||\
             savedstatus == VAL_AC_WCARD_VERIFIED ||\
-            newstatus == VAL_AC_UNSET ||\
-            newstatus == VAL_AC_UNKNOWN_ALGORITHM_LINK)\
+            newstatus == VAL_AC_UNSET)\
                 ; /* do nothing */\
         /* Any success is good */\
         else if (\
@@ -638,16 +653,16 @@ verify_next_assertion(val_context_t * ctx,
                       u_int32_t flags)
 {
     struct rrset_rec *the_set;
-    struct val_rr_rec  *the_sig;
+    struct rrset_rr  *the_sig;
     u_char       *signby_name_n;
     u_int16_t       signby_footprint_n;
     val_dnskey_rdata_t dnskey;
     int             is_a_wildcard;
-    struct val_rr_rec  *nextrr;
-    struct val_rr_rec  *keyrr;
+    struct rrset_rr  *nextrr;
+    struct rrset_rr  *keyrr;
     u_int16_t       tag_h;
-    int             is_verified = 0;
     char            name_p[NS_MAXDNAME];
+    int success = 0;
 
     if ((as == NULL) || (as->val_ac_rrset.ac_data == NULL) || (the_trust == NULL)) {
         val_log(ctx, LOG_INFO, "verify_next_assertion(): Cannot verify assertion - no data");
@@ -715,6 +730,7 @@ verify_next_assertion(val_context_t * ctx,
 
         tag_h = ntohs(signby_footprint_n);
         for (nextrr = keyrr; nextrr; nextrr = nextrr->rr_next) {
+            int             is_verified = 0;
             if (VAL_NO_ERROR != val_parse_dnskey_rdata(nextrr->rr_rdata,
                                              nextrr->rr_rdata_length,
                                              &dnskey)) {
@@ -751,13 +767,89 @@ verify_next_assertion(val_context_t * ctx,
                 nextrr->rr_status = VAL_AC_SIGNING_KEY;
             }
 
-            // if success, break
             if (is_verified) {
+
                 val_log(ctx, LOG_INFO, "verify_next_assertion(): Verified a RRSIG for %s (%s) using a DNSKEY (%d)",
                         name_p, p_type(the_set->rrs_type_h),
                         dnskey.key_tag);
-                break;
-            }
+
+                if ( as->val_ac_status == VAL_AC_TRUST ||
+                    nextrr->rr_status == VAL_AC_TRUST_POINT) {
+                    /* we've verified a trust anchor */
+                    as->val_ac_status = VAL_AC_TRUST; 
+                    val_log(ctx, LOG_INFO, "verify_next_assertion(): verification traces back to trust anchor");
+                    if (dnskey.public_key != NULL) {
+                        FREE(dnskey.public_key);
+                        dnskey.public_key = NULL;
+                    }
+                    success = 1;
+                    break;
+
+                } else if ( the_set->rrs_type_h == ns_t_dnskey && as != the_trust) {
+                    /* Check if we're trying to verify some key in the authentication chain */
+                    /* Check if we have reached our trust key */
+                    /*
+                     * If this record contains a DNSKEY, check if the DS record contains this key 
+                     * DNSKEYs cannot be wildcard expanded, so VAL_AC_WCARD_VERIFIED does not
+                     * count as a good sig
+                     * Create the link even if the DNSKEY algorithm is unknown since this 
+                     * may be the provably insecure case
+                     */
+                    /*
+                     * follow the trust path 
+                     */
+                    struct rrset_rr  *dsrec =
+                        the_trust->val_ac_rrset.ac_data->rrs_data;
+                    while (dsrec) {
+                        val_ds_rdata_t  ds;
+                        int retval;
+                        ds.d_hash = NULL;
+                        retval = val_parse_ds_rdata(dsrec->rr_rdata,
+                                       dsrec->rr_rdata_length, &ds);
+                        if(retval == VAL_NOT_IMPLEMENTED) {
+                            val_log(ctx, LOG_INFO, "verify_next_assertion(): DS hash not supported");
+                            dsrec->rr_status = VAL_AC_ALGORITHM_NOT_SUPPORTED;
+                        } else if (retval != VAL_NO_ERROR) {
+                            val_log(ctx, LOG_INFO, "verify_next_assertion(): DS parse error");
+                            dsrec->rr_status = VAL_AC_INVALID_DS;
+                        } else if (DNSKEY_MATCHES_DS(ctx, &dnskey, &ds, 
+                                    the_set->rrs_name_n, nextrr, 
+                                    &dsrec->rr_status)) {
+                            val_log(ctx, LOG_DEBUG, 
+                                    "verify_next_assertion(): DNSKEY tag (%d) matches DS tag (%d)",
+                                    (&dnskey)->key_tag,                                         
+                                    (&ds)->d_keytag);
+                            /*
+                             * the first match is enough 
+                             */
+                            nextrr->rr_status = VAL_AC_VERIFIED_LINK;
+                            dsrec->rr_status = VAL_AC_VERIFIED_LINK;
+                            FREE(ds.d_hash);
+                            ds.d_hash = NULL;
+                            if (dnskey.public_key) {
+                                FREE(dnskey.public_key);
+                                dnskey.public_key = NULL;
+                            }
+                            val_log(ctx, LOG_INFO, "verify_next_assertion(): Key links upward");
+                            success = 1;
+                            break;
+                        } else {
+                            /*
+                             * Didn't find a valid entry in the DS record set 
+                             * Not necessarily a problem, since there is no requirement that a DS be present
+                             * If none match, then we set the status accordingly. See below.
+                             */
+                            nextrr->rr_status = VAL_AC_DS_NOMATCH;
+                        } 
+
+                        if (ds.d_hash != NULL) {
+                            FREE(ds.d_hash);
+                            ds.d_hash = NULL;
+                        }
+                        dsrec = dsrec->rr_next;
+                    }
+                }
+            } 
 
             if (dnskey.public_key != NULL) {
                 FREE(dnskey.public_key);
@@ -765,106 +857,22 @@ verify_next_assertion(val_context_t * ctx,
             dnskey.public_key = NULL;
         }
 
-        if (nextrr == NULL) {
+        if (the_sig->rr_status == VAL_AC_UNSET) {
             val_log(ctx, LOG_INFO, "verify_next_assertion(): Could not link this RRSIG to a DNSKEY");
             SET_STATUS(as->val_ac_status, the_sig, VAL_AC_DNSKEY_NOMATCH);
-
-        } else if (as->val_ac_status == VAL_AC_TRUST ||
-            (is_verified && 
-                     nextrr->rr_status == VAL_AC_TRUST_POINT)) {
-            /* we've verified a trust anchor */
-
-            as->val_ac_status = VAL_AC_TRUST; 
-            val_log(ctx, LOG_INFO, "verify_next_assertion(): verification traces back to trust anchor");
-            if (dnskey.public_key != NULL) {
-                FREE(dnskey.public_key);
-                dnskey.public_key = NULL;
-            }
-            return;
-            
-        /* else check if we're trying to verify some key in the authentication chain */
-        } else if ( the_set->rrs_type_h == ns_t_dnskey && 
-                    as != the_trust &&
-                    as->val_ac_status != VAL_AC_TRUST &&
-                     (is_verified || the_sig->rr_status == VAL_AC_ALGORITHM_NOT_SUPPORTED)) {
-
-            /* Check if we have reached our trust key */
-            
-            /*
-             * If this record contains a DNSKEY, check if the DS record contains this key 
-             * DNSKEYs cannot be wildcard expanded, so VAL_AC_WCARD_VERIFIED does not
-             * count as a good sig
-             * Create the link even if the DNSKEY algorithm is unknown since this 
-             * may be the provably insecure case
-             */
-            /*
-             * follow the trust path 
-             */
-            struct val_rr_rec  *dsrec =
-                the_trust->val_ac_rrset.ac_data->rrs_data;
-            while (dsrec) {
-                val_ds_rdata_t  ds;
-                int retval = val_parse_ds_rdata(dsrec->rr_rdata,
-                                   dsrec->rr_rdata_length, &ds);
-                if(retval == VAL_NOT_IMPLEMENTED) {
-                    val_log(ctx, LOG_INFO, "verify_next_assertion(): DS hash not supported");
-                    dsrec->rr_status = VAL_AC_ALGORITHM_NOT_SUPPORTED;
-                } else if (retval != VAL_NO_ERROR) {
-                    val_log(ctx, LOG_INFO, "verify_next_assertion(): DS parse error");
-                    dsrec->rr_status = VAL_AC_INVALID_DS;
-                } else {
-
-                    if (DNSKEY_MATCHES_DS(ctx, &dnskey, &ds, 
-                            the_set->rrs_name_n, nextrr, 
-                            &dsrec->rr_status)) {
-
-                        val_log(ctx, LOG_DEBUG, 
-                                "verify_next_assertion(): DNSKEY tag (%d) matches DS tag (%d)",
-                                (&dnskey)->key_tag,                                         
-                                (&ds)->d_keytag);
-                        if (is_verified)
-                            nextrr->rr_status = VAL_AC_VERIFIED_LINK;
-                        else
-                            nextrr->rr_status = VAL_AC_UNKNOWN_ALGORITHM_LINK;
-
-                        FREE(ds.d_hash);
-                        ds.d_hash = NULL;
-                        if (dnskey.public_key) {
-                            FREE(dnskey.public_key);
-                            dnskey.public_key = NULL;
-                        }
-                        /*
-                        * the first match is enough 
-                        */
-                        val_log(ctx, LOG_INFO, "verify_next_assertion(): Key links upward");
-                        return;
-                    }
-
-                    FREE(ds.d_hash);
-                    ds.d_hash = NULL;
-                }
-
-                dsrec = dsrec->rr_next;
-            }
-
-            /*
-             * Didn't find a valid entry in the DS record set 
-             * Not necessarily a problem, since there is no requirement that a DS be present
-             * If none match, then we set the status accordingly. See below.
-             */
-            nextrr->rr_status = VAL_AC_DS_NOMATCH;
         }
 
-        if (dnskey.public_key != NULL) {
-            FREE(dnskey.public_key);
-            dnskey.public_key = NULL;
+        /* Continue checking only if we want to verify all signatures */
+        if (success && !(flags & VAL_QUERY_CHECK_ALL_RRSIGS)) {
+            break;
         }
     }
-
-    /* If we reach here and we're a keyset, we either didn't verify the keyset or
-       didn't verify the link from the key to the DS 
+        
+    /* 
+     * If we reach here and we're a keyset, we either didn't verify the keyset or
+     * didn't verify the link from the key to the DS 
      */ 
-    if (the_set->rrs_type_h == ns_t_dnskey){
+    if (!success && the_set->rrs_type_h == ns_t_dnskey){
         as->val_ac_status = VAL_AC_NO_LINK;
     }
 }
