@@ -591,6 +591,7 @@ val_res_search(val_context_t * context, const char *dname, int class_h,
     int             retval = -1;
     char           *dot, *search, *pos;
     char            buf[NS_MAXDNAME];
+    int             last_err;
     val_context_t *ctx = NULL;
     SET_LAST_ERR(NO_RECOVERY);
     //SET_LAST_ERR(NETDB_INTERNAL);
@@ -610,7 +611,7 @@ val_res_search(val_context_t * context, const char *dname, int class_h,
             dname, p_class(class_h), p_type(type), p_val_err(VAL_BAD_ARGUMENT));
         errno = EINVAL;
         retval = -1;
-        goto err;
+        goto done;
     }
 
     /*
@@ -636,11 +637,24 @@ val_res_search(val_context_t * context, const char *dname, int class_h,
             snprintf(buf, sizeof(buf), "%s.%s", dname, search);
             retval = val_res_query(ctx, buf, class_h, type, answer, anslen,
                                    val_status);
-            if ((retval >0) ||
-                ((retval == -1) && (GET_LAST_ERR() != HOST_NOT_FOUND))) {
+            /*
+             * Continue looping if we don't have a valid result
+             * and we haven't run into any hard error.
+             */
+            if (retval < 0) {
+                last_err = GET_LAST_ERR();
+                if (last_err != HOST_NOT_FOUND &&/* name does not exist */
+                    last_err != TRY_AGAIN) {/* DNS error */
+                    if (save)
+                        free(save);
+                    goto done;
+                }
+                /* need to re-try */
+            } else {
+                /* success */
                 if (save)
                     free(save);
-                goto err;
+                goto done;
             }
 
             if (*pos)
@@ -656,7 +670,7 @@ val_res_search(val_context_t * context, const char *dname, int class_h,
     retval = val_res_query(ctx, dname, class_h, type, answer, anslen,
                            val_status);
 
-err:
+done:
     CTX_UNLOCK_POL(ctx);
 
     return retval;
